#include "stdafx.h"
#include "resource.h"
#include "Action.h"


Action::Action(const char *path)
{
	MSXML2::IXMLDOMDocumentPtr dom;
	MSXML2::IXMLDOMDocument2Ptr document;
	MSXML2::IXMLDOMNodePtr node;
	MSXML2::IXMLDOMNodePtr rootNode;

	try
	{
		CString when;
		_variant_t val;

		dom.CreateInstance(__uuidof(MSXML2::DOMDocument));
		document = dom;
		document->put_async(VARIANT_FALSE);
		document->setProperty(_bstr_t("SelectionLanguage"), _variant_t("XPath"));
		document->load(path);
		rootNode = document->selectSingleNode("/action");
		val = rootNode->selectSingleNode("uuid")->firstChild->nodeValue;
		SetGuidStr((const char*) _bstr_t(val));
		val = rootNode->selectSingleNode("name")->firstChild->nodeValue;
		m_name = (const char*) _bstr_t(val);
		val = rootNode->selectSingleNode("server")->firstChild->nodeValue;
		m_server = (const char*) _bstr_t(val);
		val = rootNode->selectSingleNode("port")->firstChild->nodeValue;
		m_port = (unsigned short)(short)(val);
		val = rootNode->selectSingleNode("description")->firstChild->nodeValue;
		m_description = (const char *)_bstr_t(val);
		val = rootNode->selectSingleNode("when")->firstChild->nodeValue;
		when = (const char *)_bstr_t(val);
		m_when = StrToTime(when);
		val = rootNode->selectSingleNode("duration")->firstChild->nodeValue;
		m_howLong = atoi((const char *)_bstr_t(val));
	}
	catch(_com_error e)
	{
		CString s = "Errore Parsificazione Action file: ";
		s += path;
		AfxMessageBox(s);
	}
}

Action::Action(const char *name, const char *server, unsigned short port, 
			   const char *description, CTime when, int howLong) :
	m_name(name), m_server(server), m_port(port), 
	m_description(description), m_when(when), m_howLong(howLong)
{
	UuidCreate(&m_guid);
}

const CString& Action::GetName() const
{
	return m_name;
}

const CString& Action::GetServer() const
{
	return m_server;
}

unsigned short Action::GetPort() const
{
	return m_port;
}

CString Action::GetPortStr() const
{
	CString s;
	s.Format("%hu", m_port);
	return s;
}

const CString& Action::GetDescription() const
{
	return m_description;
}

CTime Action::GetWhen() const
{
	return m_when;
}

int Action::GetHowLong() const
{
	return m_howLong;
}

CString Action::GetHowLongStr() const
{
	CString s;
	s.Format("%d", m_howLong);
	return s;
}

CString Action::GetWhenStr() const
{
	return m_when.Format("%d/%m/%Y %H:%M");
}

void Action::MakeLove(const char* dir) const
{
	MSXML2::IXMLDOMDocumentPtr document;
	MSXML2::IXMLDOMNodePtr node;
	MSXML2::IXMLDOMNodePtr rootNode;
	CString path;
	char bufTmp[128];

	document.CreateInstance(__uuidof(MSXML2::DOMDocument));
	document->put_async(VARIANT_FALSE);
	node = document->createProcessingInstruction("xml", "version='1.0'");
	document->appendChild(node);
	rootNode = document->createElement("action");
	document->appendChild(rootNode);
	node = document->createElement("uuid");
	node->appendChild(document->createTextNode((const char *)GetGuidStr()));
	rootNode->appendChild(node);
	node = document->createElement("name");
	node->appendChild(document->createTextNode((const char *)m_name));
	rootNode->appendChild(node);
	node = document->createElement("server");
	node->appendChild(document->createTextNode((const char *)m_server));
	rootNode->appendChild(node);
	node = document->createElement("port");
	node->appendChild(document->createTextNode(itoa((int)m_port, bufTmp, 10)));
	rootNode->appendChild(node);
	node = document->createElement("description");
	node->appendChild(document->createTextNode((const char *)m_description));
	rootNode->appendChild(node);
	node = document->createElement("when");
	node->appendChild(document->createTextNode((const char *)GetWhenStr()));
	rootNode->appendChild(node);
	node = document->createElement("duration");
	node->appendChild(document->createTextNode((const char *)GetHowLongStr()));
	rootNode->appendChild(node);
	path = CString(dir);
	if (path.Right(1) != "\\")
		path += "\\";
	path += m_when.Format("%Y_%m_%d") + "_uuid_" + GetGuidStr() + ".love"; 
	document->save((const char *)path);
}

void Action::SetGuidStr(const char *str)
{
	if (RPC_S_OK != UuidFromString((unsigned char *)str, 
		const_cast<GUID *>(&m_guid)))
		AfxMessageBox("Errore stringa uuid");
}

CString Action::GetGuidStr() const
{
	unsigned char *str;
	CString s;
	
	if (RPC_S_OK == UuidToString(const_cast<GUID *>(&m_guid), &str))
	{
		s = str;
		RpcStringFree(&str);
	}
	else
	{
		AfxMessageBox(IDP_INTERNAL_ERROR);
	}
	return s;
}

CTime Action::StrToTime(const char *strTime)
{
	int day;
	int month;
	int year;
	int hour;
	int minute;
	CString s(strTime);
	CString t;

	// I did write the date, therefore I read my format
	day = atoi((s.Left(2)));
	s = s.Mid(3);
	month = atoi((s.Left(2)));
	s = s.Mid(3);
	year = atoi((s.Left(4)));
	s = s.Mid(5);
	hour = atoi((s.Left(2)));
	s = s.Mid(3);
	minute = atoi((s.Left(2)));
	return CTime(year, month, day, hour, minute, 0);
}

