/******************************************************************************
*
* AntiMafia
* Copyright (C) 2002 by [epidemiC] AntiMafia
* Newtella
* Copyright (C) 2000 by Newtella Inc.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
******************************************************************************/


// AntiMafiaSheet.cpp : implementation file
//

#include "stdafx.h"
#include <direct.h>
#include "AntiMafia.h"
#include "Core\common.h"
#include "Core\Buffer.h"
#include "Core\Object.h"
#include "Core\List.h"
#include "Core\String.h"
#include "Core\ErrorInfo.h"
#include "Core\FileWorker.h"
#include "Core\Connection.h"
#include "Core\GnutellaWorker.h"
#include "Core\Base.h"
#include "ConnectionPage.h"
#include "ConfigPage.h"
#include "AboutPage.h"
#include "ActionPage.h"
#include "NewAction.h"

#include "AntiMafiaSheet.h"
#include "ActionManager.h"


/////////////////////////////////////////////////////////////////////////////
// CAntiMafiaSheet

IMPLEMENT_DYNAMIC(CAntiMafiaSheet, CPropertySheet)

CAntiMafiaSheet::CAntiMafiaSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
}

CAntiMafiaSheet::CAntiMafiaSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
}

CAntiMafiaSheet::~CAntiMafiaSheet()
{
}


BEGIN_MESSAGE_MAP(CAntiMafiaSheet, CPropertySheet)
	//{{AFX_MSG_MAP(CAntiMafiaSheet)
	ON_WM_KEYDOWN()
	ON_WM_SYSCOMMAND()
	ON_WM_SIZE()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAntiMafiaSheet message handlers

LRESULT CAntiMafiaSheet::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	static BOOL errorsShown = TRUE;
	static BOOL ignoreErrors= TRUE;
	CRect ErrorRect, SheetRect;
	CRect wndRect;

	switch (message)
	{
	case WM_COMMAND:
		if (LOWORD(wParam) == IDC_ADVANCED)
		{
			ConfigPage.OnAdvanced();
			return(TRUE);
		}
		break;
	case WM_KEYDOWN:
		if (wParam == VK_F1)
			SetActivePage(0);
		if (wParam == VK_F2)
			SetActivePage(1);
		if (wParam == VK_F3)
			SetActivePage(2);
		if (wParam == VK_F4)
			SetActivePage(3);
		break;
	case WM_TIMER:
		if (wParam == TIMER_SECOND)
			gnutella->EverySecond();
		if (wParam == TIMER_MINUTE)
			gnutella->EveryMinute();
		if (wParam == TIMER_TENMINUTES)
			gnutella->EveryTenMinutes();
		if (WaitForSingleObject(gnutellanetEvent, 0) == WAIT_OBJECT_0)
		{
			if (ConnectionPage.UpdatePage())
				ResetEvent(gnutellanetEvent);
		}
		
		if (WaitForSingleObject(getEvent, 0) == WAIT_OBJECT_0)
		{
			if (ActionManager::Instance()->OnGetEvent())
				ResetEvent(getEvent);
		}
		break;
	default:
		if (message == CHANGED_FOUND_FILES)
		{
			ActionManager *pam = ActionManager::Instance();
			int foundFiles = gnutella->foundFiles.Count();
			FileInfo *pfi;

			pfi = (FileInfo*)gnutella->foundFiles.GetFirst(FALSE);
			while(pfi = (FileInfo*)gnutella->foundFiles.GetNext())
				pam->HandleFile(pfi);
		}
		break ;

	}
	
	return CPropertySheet::WindowProc(message, wParam, lParam);
}

BOOL CAntiMafiaSheet::OnInitDialog()
{

// Create Events
	getEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	sendEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	gnutellanetEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
// Add minimize and maximize buttons
	DWORD dw = GetStyle() ;
	ModifyStyle(dw, WS_OVERLAPPEDWINDOW);
// Add about menu
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);
	CMenu* pSysMenu = GetSystemMenu(FALSE);
	CString strAboutMenu;
	strAboutMenu.LoadString(IDS_ABOUTBOX);
	if (!strAboutMenu.IsEmpty())
	{
		pSysMenu->InsertMenu(SC_MOVE, MF_STRING | MF_BYCOMMAND, SC_RESTORE, "Restore") ;
		pSysMenu->InsertMenu(SC_CLOSE, MF_STRING | MF_BYCOMMAND, SC_SIZE, "Size") ;
		pSysMenu->InsertMenu(SC_CLOSE, MF_STRING | MF_BYCOMMAND, SC_MINIMIZE, "Minimize") ;
		pSysMenu->InsertMenu(SC_CLOSE, MF_STRING | MF_BYCOMMAND, SC_MAXIMIZE, "Maximize") ;
		pSysMenu->InsertMenu(SC_CLOSE, MF_SEPARATOR | MF_BYCOMMAND) ;
	}
// Initialize Gnutella
	if (!gnutella->InitFiles())
	{
		EndDialog(1);
		return(FALSE);
	}
	if (!gnutella->InitSockets())
	{
		EndDialog(1);
		return(FALSE);
	}
	if (!gnutella->InitMessageWindow())
	{
		EndDialog(1);
		return(FALSE);
	}
	if (!gnutella->InitListener())
	{
		do
		{
			sprintf(ods, "Could not load AntiMafia at port %d.\nRetry at port %d?", ntohs(global.options.listenPort), ntohs(global.options.listenPort)+1);
			if (MessageBox(ods, "Error", MB_ICONSTOP | MB_RETRYCANCEL) == IDCANCEL)
			{
				EndDialog(1);
				return(FALSE);
			}
			else
				global.options.listenPort = htons(ntohs(global.options.listenPort) + 1);
		} while (!gnutella->InitListener());
	}
	// a time server will work...
	gnutella->GetTime(0);
	gnutella->GetTime(1);
	gnutella->GetTime(2);
// Add icons
	SetIcon(AntiMafiaIcon, TRUE);
	SetIcon(AntiMafiaIcon, FALSE);
	CenterWindow();
// Add the Advanced Config Button
	font.CreatePointFont(90, "MS Sans Serif");
	AdvancedConfigButton.Create("Configure", WS_CHILD | WS_VISIBLE, CRect(21, 445, 150, 468), this, IDC_ADVANCED);
	AdvancedConfigButton.SetFont(&font);
//Remove the Help Button
	GetDlgItem(IDHELP)->ShowWindow(SW_HIDE);
	GetDlgItem(IDHELP)->EnableWindow(FALSE);
// Remove the OK button
	GetDlgItem(IDOK)->ShowWindow(SW_HIDE);
	GetDlgItem(IDOK)->EnableWindow(FALSE);
	GetDlgItem(IDCANCEL)->SetWindowText("Quit AntiMafia");
// Start timers
	SetTimer(TIMER_SECOND, 1000, NULL);
	SetTimer(TIMER_MINUTE, 60000, NULL);
	SetTimer(TIMER_TENMINUTES, 600000, NULL);

// Call the default handler
	BOOL bResult = CPropertySheet::OnInitDialog();
	
// Add the Menu
	MainMenu.LoadMenu(IDR_MAINMENU) ;
	SetMenu(&MainMenu) ;

// Add the AntiMafia Logo
	AntiMafiaLogo.LoadBitmap(IDB_LOGO) ;
	AntiMafiaLogoDC.CreateCompatibleDC(GetWindowDC()) ;
	AntiMafiaLogoDC.SelectObject(AntiMafiaLogo) ;
	AntiMafiaLogoDC.SelectStockObject(LTGRAY_BRUSH) ;
	AntiMafiaLogoDC.ExtFloodFill(1, 1, 0x00FFFFFF, FLOODFILLSURFACE) ;

// Reposition the Exit and Help buttons
	GetDlgItem(IDCANCEL)->SetWindowPos(NULL, 364, 445, 100, 23, 0) ;
	GetDlgItem(IDHELP)->SetWindowPos(NULL, 284, 445, 70, 23, 0) ;

// Resize window to last known size and position
	ShowWindow(SW_SHOWNA) ;
	SetWindowPos(NULL, global.WindowX, global.WindowY, global.WindowCX, global.WindowCY, SWP_NOZORDER) ;
	if (global.WindowX == -1 && global.WindowY == -1) CenterWindow() ;
	if (global.WindowZoomed) ShowWindow(SW_SHOWMAXIMIZED) ;

	gnutella->EveryTenMinutes();

	return bResult;
}


void CAntiMafiaSheet::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar == VK_ESCAPE)
		EndDialog(IDCANCEL);
	
	CPropertySheet::OnKeyDown(nChar, nRepCnt, nFlags);
}


BOOL CALLBACK SizeChildrenCallbackProc(HWND hwnd, LPARAM lParam)
{
 
	RECT rect ;
	char szClassName[MAX_PATH] ;
	char szParentClassName[MAX_PATH] ;
	int ID ;
	short dx = LOWORD(lParam), dy = HIWORD(lParam) ;
	HWND hWndParent = ::GetParent(hwnd) ;
	
	::GetWindowRect(hwnd, &rect) ;

	POINT tp ;
	tp.x = rect.left ;
	tp.y = rect.top ;
	::ScreenToClient(hWndParent, &tp) ;
	rect.left = tp.x;
	rect.top = tp.y;

	tp.x = rect.right ;
	tp.y = rect.bottom ;
	::ScreenToClient(hWndParent, &tp) ;
	rect.right = tp.x;
	rect.bottom = tp.y;

	::GetClassName(hwnd, szClassName, MAX_PATH) ;
	::GetClassName(hWndParent, szParentClassName, MAX_PATH) ;
	ID = ::GetDlgCtrlID(hwnd) ;

	if (ID == IDC_STATE || ID == IDC_STATICTIME || ID == IDC_FILESFOUND || ID == IDC_STATICFILESFOUND) ;
	else if (!strcmp(szParentClassName, "AfxFrameOrView42sd") || !strcmp(szParentClassName, "AfxMDIFrame42sd") || !strcmp(szParentClassName, "SysDateTimePick32")) ;
	else if (ID == IDC_STATICRESULTS || ID == IDC_ERRORLISTBOX || !strcmp(szClassName, "SysListView32") 
		|| !strcmp(szClassName, "#32770") || !strcmp(szClassName, "SysTabControl32")
		|| !strcmp(szClassName, "AfxFrameOrView42sd"))
	{
		::SetWindowPos(hwnd, NULL, rect.left, rect.top, rect.right-rect.left+dx, rect.bottom-rect.top+dy, SWP_NOZORDER) ;
	}
	else if (ID == IDC_SEARCHSTRING)
	{
		::SetWindowPos(hwnd, NULL, rect.left, rect.top, rect.right-rect.left+dx, rect.bottom-rect.top, SWP_NOZORDER) ;
	}
	else if (ID == IDC_SEARCH || ID == IDC_CLEARSEARCHLIST || ID == IDC_DISPLAYALL || ID == IDC_LOGOBUTTON)
	{
		::SetWindowPos(hwnd, NULL, rect.left+dx, rect.top, rect.right-rect.left, rect.bottom-rect.top, SWP_NOZORDER) ;
	}
	else if (!strcmp(szClassName, "SysHeader32"))
	{
		::SetWindowPos(hwnd, NULL, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, SWP_NOZORDER) ;
	}
	else
	{
		::SetWindowPos(hwnd, NULL, rect.left+dx, rect.top+dy, rect.right-rect.left, rect.bottom-rect.top, SWP_NOZORDER) ;
	}
		
	return true ;	

}





	
void CAntiMafiaSheet::OnSize(UINT nType, int cx, int cy) 
{

	RECT oldRect;
	short dx, dy ;

	CPropertySheet::OnSize(nType, cx, cy);

	if (IsWindowVisible() && (nType == SIZE_MAXIMIZED || nType == SIZE_RESTORED))
	{

		GetDlgItem(IDCANCEL)->GetWindowRect(&oldRect) ;
		oldRect.right += 4 ;
		oldRect.bottom += 5 ;
		ScreenToClient(&oldRect) ;
		
		if (cx < 467) dx = 0 ;
		else dx = (short)(cx - oldRect.right) ;

		if (cy < 450) dy = 0 ;
		else dy = (short)(cy - oldRect.bottom) ;

		if (dx || dy)
		{

			::EnumChildWindows(m_hWnd, SizeChildrenCallbackProc, MAKELONG(dx, dy)) ;
		}

		::InvalidateRect(m_hWnd, NULL, TRUE) ;

	}

}




BOOL CAntiMafiaSheet::DestroyWindow() 
{
	RECT rect ;

	global.WindowZoomed = IsZoomed() ;

	if (!global.WindowZoomed)
	{
		GetWindowRect(&rect) ;
		global.WindowX = rect.left ;
		global.WindowY = rect.top ;
		global.WindowCX = rect.right-rect.left ;
		global.WindowCY = rect.bottom-rect.top ;
	}

	return CPropertySheet::DestroyWindow();
}





void CAntiMafiaSheet::OnPaint() 
{
	CPaintDC dc(this) ;

	int dx, cx, cy ;
	RECT rect ;
	GetWindowRect(&rect) ;
	cx = rect.right-rect.left-4 ;
	cy = rect.bottom-rect.top-4 ;
	dx = (cx > 467 ? cx-130 : 337) ;
	// GetWindowDC()->BitBlt(dx, 22, cx-dx, (cy > 24 ? 44 : cy+20), &AntiMafiaLogoDC, 0, 0, SRCCOPY) ;
	ValidateRect(CRect(dx-4, 0, cx-4, 24)) ;
}

