/******************************************************************************
*
* AntiMafia
* Copyright (C) 2002 by [epidemiC] AntiMafia
* Newtella
* Copyright (C) 2000 by Newtella Inc.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
******************************************************************************/

#include "..\stdafx.h"
#include <string.h>
#include "common.h"
#include "Buffer.h"

Buffer::Buffer()
{
	data = NULL; length = 0; realLength = 0;
	//owner = 0; use = 0; count = 0;
	pageSize = 0;
}

Buffer::~Buffer()
{
	if (data)
	{
		delete [] data;
		data = NULL;
	}
}

BOOL Buffer::Alloc(int l, LPCSTR newdata)
{
	if (data)
	{
		delete [] data;
		data = NULL;
	}

	int rl;
	if (pageSize)
		rl = (l/pageSize + 1) * pageSize;
	else
		rl = l;

	data = new char[rl];
	if (!data)
		return(FALSE);

	length = l;
	realLength = rl;

	if (newdata)
		memcpy(data, newdata, length);

	return(TRUE);
}

BOOL Buffer::ReAlloc(int l, BOOL delta)
{
	int newlength;

	if (delta)
		newlength = length + l;
	else
		newlength = l;
	if (newlength < 0)
		return(FALSE);
	if (newlength == 0)
	{
		if (data)
		{
			delete [] data;
			data = NULL;
		}
		length = realLength = 0;
		return(TRUE);
	}

	int rl;
	if (pageSize)
		rl = (newlength/pageSize + 1) * pageSize;
	else
		rl = newlength;

	if (data)
	{
		if (rl != realLength)
		{
			char *copybuf = new char[rl];
			if (!copybuf)
				return(FALSE);
			memcpy(copybuf, data, MIN(length, newlength));
			delete [] data;
			data = copybuf;
		}
	}
	else
	{
		data = new char[rl];
		if (!data)
			return(FALSE);
	}
	length = newlength;
	realLength = rl;

	return(TRUE);
}

BOOL Buffer::ReAllocAtEnd(int l, BOOL delta)
{
	int newlength;

	if (delta)
		newlength = length + l;
	else
		newlength = l;
	if (newlength < 0)
		return(FALSE);
	if (newlength == 0)
	{
		if (data)
		{
			delete [] data;
			data = NULL;
		}
		length = realLength = 0;
		return(TRUE);
	}

	int rl;
	if (pageSize)
		rl = (newlength/pageSize + 1) * pageSize;
	else
		rl = newlength;

	if (data)
	{
		if (rl != realLength)
		{
			char *copybuf = new char[rl];
			if (!copybuf)
				return(FALSE);
			if (newlength < length)
				memcpy(copybuf, data + length - newlength, newlength);
			else
				memcpy(copybuf + newlength - length, data, length);
			delete [] data;
			data = copybuf;
		}
	}
	else
	{
		data = new char[rl];
		if (!data)
			return(FALSE);
	}
	length = newlength;
	realLength = rl;

	return(TRUE);
}

BOOL Buffer::SetPageSize(int ps)
{
	pageSize = ps;
	return(TRUE);
}

BOOL Buffer::Free()
{
	if (!pageSize)
	{
		if (data)
		{
			delete [] data;
			data = NULL;
		}
		realLength = 0;
	}
	length = 0;
	return(TRUE);
}
