/******************************************************************************
*
* AntiMafia
* Copyright (C) 2002 by [epidemiC] AntiMafia
* Newtella
* Copyright (C) 2000 by Newtella Inc.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
******************************************************************************/



#include "..\stdafx.h"
#include <stdio.h>
#include <io.h>
#include "common.h"
#include "Buffer.h"
#include "Object.h"
#include "String.h"
#include "ErrorInfo.h"
#include "HTTPHeader.h"
#include "List.h"
#include "FileWorker.h"
#include "Connection.h"
#include "GnutellaWorker.h"
#include "Base.h"





Packet::Packet()
{
	time = 0;
}

Packet::~Packet()
{
}

ConnectionInfo::ConnectionInfo()
{
	responseCode = RC_INVALID;

	lastTime = GetTickCount();
	state = GS_UNUSED; type = CT_NONE;
	memset(&address, 0, sizeof(struct sockaddr_in));
	sent = 0; received = 0; dropped = 0;
	id = 0; relatedID = 0;
	temporary = FALSE;
	transmitted = 0;
	headerTransmitted = FALSE;
	speed = 0;
	speedData = 0;
	speedTime = GetTickCount();
	memset(&speeds, 0, sizeof(speeds));
	speedIndex = 0;
	limitData = 0;
}

ConnectionInfo::~ConnectionInfo()
{
}

Connection::Connection()
{
	ConnectionInfo::ConnectionInfo();
	socket = INVALID_SOCKET;
	b = NULL;
	bufferLength = 0;
	packets.SetMaxNodes(100);
}

Connection::~Connection()
{
	CloseSocket();
	ResetBuffer();
}

BOOL Connection::SetBandwidthLimit(int kbytes)
{
	limitData = kbytes;
	return(TRUE);
}

BOOL Connection::ComputeSpeed(DWORD timenow)
{
	if (speedTime == timenow)
		return(FALSE);
	if (state != GS_CONNECTED)
		return(TRUE);
	speedIndex = (speedIndex + 1) & 7;
	speeds[speedIndex] = speedData*1000/(timenow-speedTime);
	speedTime = timenow;
	speedData = 0;
	speed = (speeds[0] + speeds[1] + speeds[2] + speeds[3] +
			speeds[4] + speeds[5] + speeds[6] + speeds[7]) >> 3;
	return(TRUE);
}

int Connection::SendPacket(const char *buf, int len)
{
	int rval = send(socket, buf, len, 0);
	if (rval == SOCKET_ERROR)
	{
		dropped++;
		return(SOCKET_ERROR);
	}
	sent++;
	global.stats.sent++;
	speedData += len;
	return(rval);
}

int Connection::Send(const char *buf, int len)
{
	int rval = send(socket, buf, len, 0);
	if (rval == SOCKET_ERROR)
		return(SOCKET_ERROR);
	speedData += len;
	return(rval);
}

int Connection::ReceivePacket(char *buf, int len)
{
	int rval = recv(socket, buf, len, 0);
	if (rval == SOCKET_ERROR)
	{
		dropped++;
		return(SOCKET_ERROR);
	}
	received++;
	global.stats.received++;
	speedData += len;
	return(rval);
}

int Connection::Receive(char *buf, int len)
{
	int rval = recv(socket, buf, len, 0);
	if (rval == SOCKET_ERROR)
		return(SOCKET_ERROR);
	speedData += len;
	return(rval);
}

BOOL Connection::ResetBuffer()
{
	buffer.Free();
	b = NULL;
	return(TRUE);
}

BOOL Connection::CloseSocket()
{
	int rval;
	rval = closesocket(socket);
	if (rval == SOCKET_ERROR)
		return(FALSE);
	socket = INVALID_SOCKET;
	
	return(TRUE);
}

BOOL Connection::Serialize(FILE *file, BOOL store)
{
	const int serializeVersion=1;
	int fileVersion;

	char *tb = 0;

	if (store)
	{
		STORE_VAR(serializeVersion);

		STORE_VAR(id);
		STORE_VAR(relatedID);
		STORE_VAR(state); STORE_VAR(type);
		STORE_VAR(responseCode);
		STORE_VAR(sent); STORE_VAR(received); STORE_VAR(dropped);
		STORE_VAR(lastTime);
		STORE_VAR(temporary);
		STORE_VAR(speed);
		STORE_VAR(tb);
		STORE_VAR(socket);
		STORE_VAR(address);
		STORE_VAR(transmitted);
		STORE_VAR(headerTransmitted);
		STORE_VAR(speeds);
		STORE_VAR(speedIndex);
		STORE_VAR(speedData);
		STORE_VAR(speedTime);
		STORE_VAR(limitData);
		STORE_SUBCLASS(fi);
	}
	else
	{
		CHECK_VERSION();

		LOAD_VAR(id);
		LOAD_VAR(relatedID);
		LOAD_VAR(state); LOAD_VAR(type);
		LOAD_VAR(responseCode);
		LOAD_VAR(sent); LOAD_VAR(received); LOAD_VAR(dropped);
		LOAD_VAR(lastTime);
		LOAD_VAR(temporary);
		LOAD_VAR(speed);
		LOAD_VAR(b);
		LOAD_VAR(socket);
		LOAD_VAR(address);
		LOAD_VAR(transmitted);
		LOAD_VAR(headerTransmitted);
		LOAD_VAR(speeds);
		LOAD_VAR(speedIndex);
		LOAD_VAR(speedData);
		LOAD_VAR(speedTime);
		LOAD_VAR(limitData);
		LOAD_SUBCLASS(fi);
	}
	return(TRUE);
serializeFailed:
	return(FALSE);
}

BOOL Connection::GetIp(unsigned char *ip) const
{
	sockaddr_in name;
	int size = sizeof(name);

	if (SOCKET_ERROR == getpeername(socket, (sockaddr*) &name, &size))
		return FALSE;
	memcpy(ip, &name.sin_addr, 4);
	return TRUE;
}	