/******************************************************************************
*
* AntiMafia
* Copyright (C) 2002 by [epidemiC] AntiMafia
* Newtella
* Copyright (C) 2000 by Newtella Inc.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
******************************************************************************/


#include "..\stdafx.h"
#include <stdio.h>
#include <io.h>
#include <process.h>
#include <direct.h>
#include "common.h"
#include "Buffer.h"
#include "Object.h"
#include "String.h"
#include "ErrorInfo.h"
#include "HTTPHeader.h"
#include "List.h"
#include "FileWorker.h"
#include "Connection.h"
#include "GnutellaWorker.h"
#include "Base.h"
#include "../ActionManager.h"
#include "../Logger.h"
#include "../MyTime.h"

UINT INCOMING_ACCEPT;
UINT GNUTELLANET_CONNECTED;
UINT GET_CONNECTED;
UINT PUSH_CONNECTED;
UINT GNUTELLANET_DATA;
UINT SEND_WRITE;
UINT GET_READ;
UINT GETTIME_CONNECTED;
UINT GETTIME_READ;

HANDLE getEvent, sendEvent, gnutellanetEvent;

FileWorker GnutellaWorker::files;
HANDLE GnutellaWorker::filesMutex;
List GnutellaWorker::resolveHosts;
HANDLE GnutellaWorker::resolveHostsMutex, GnutellaWorker::resolveHostsAbort;

GnutellaWorker::GnutellaWorker()
{

	char tempbuf[MAX_TEMPBUF_LEN] ;
// Initialize mutexes
	filesMutex = CreateMutex(NULL, FALSE, "filesMutex");
	resolveHostsMutex = CreateMutex(NULL, FALSE, "resolveHostsMutex");
	resolveHostsAbort = CreateMutex(NULL, FALSE, "resolveHostsAbort");
// Set vars
	messageWindow = NULL;
	serverSocket = INVALID_SOCKET;
	memset(&currentSearchGUID, 0, sizeof(GLUID));
	memset(&wsaData, 0, sizeof(wsaData));
	lastPing = 0;
// Register used window messages
	INCOMING_ACCEPT = RegisterWindowMessage("INCOMING_ACCEPT");
	GNUTELLANET_CONNECTED = RegisterWindowMessage("GNUTELLANET_CONNECTED");
	GET_CONNECTED = RegisterWindowMessage("GET_CONNECTED");
	PUSH_CONNECTED = RegisterWindowMessage("PUSH_CONNECTED");
	GNUTELLANET_DATA = RegisterWindowMessage("GNUTELLANET_DATA");
	SEND_WRITE = RegisterWindowMessage("SEND_WRITE");
	GET_READ = RegisterWindowMessage("GET_READ");
	GETTIME_CONNECTED = RegisterWindowMessage("GETTIME_CONNECTED");
	GETTIME_READ = RegisterWindowMessage("GETTIME_READ");
// Set max nodes for all lists
	gnutellanetConnections.SetMaxNodes(300);
	downloadConnections.SetMaxNodes(300);
	uploadConnections.SetMaxNodes(300);
	foundFiles.SetMaxNodes(8192);
	receivedResults.SetMaxNodes(8192);
	hosts.SetMaxNodes(1000);
	requestedPushes.SetMaxNodes(100);
	pingRoutes.SetMaxNodes(1000);
	searchRoutes.SetMaxNodes(1000);
	pushRoutes.SetMaxNodes(1000);
// Load serialized data
	WaitForSingleObject(filesMutex, INFINITE);
	files.fileList.SetDataType(DATATYPE_FILEINFO);


	ReleaseMutex(filesMutex);

	foundFiles.SetDataType(DATATYPE_FILEINFO);
	strcpy(tempbuf, global.options.programDir) ;
	strcat(tempbuf, "SearchResults.dat") ;
	FILE *foundfile = fopen(tempbuf, "rb");
	if (foundfile)
	{
		foundFiles.Serialize(foundfile);
		fclose(foundfile);
	}
	global.searchFileCount = 0 ;
	foundFiles.GetFirst(FALSE) ;
	do
	{
		foundFiles.SetCurrentDescriptor(global.searchFileCount++) ;
	} while (foundFiles.GetNext()) ;

	hosts.SetDataType(DATATYPE_HOSTINFO);
	strcpy(tempbuf, global.options.programDir) ;
	strcat(tempbuf, "Hosts.dat") ;
	FILE *hostfile = fopen(tempbuf, "rb");
	if (hostfile)
	{
		hosts.Serialize(hostfile);
		fclose(hostfile);
		DWORD timenow = GetTickCount();
		HostInfo *hi;
		hosts.GetFirst();
		while (hi = (HostInfo *)hosts.GetNext())
			hi->time = timenow;
	}
	downloadConnections.SetDataType(DATATYPE_CONNECTION);
	strcpy(tempbuf, global.options.programDir) ;
	strcat(tempbuf, "Partials.dat") ;
	FILE *partfile = fopen(tempbuf, "rb");
	if (partfile)
	{
		downloadConnections.Serialize(partfile);
		fclose(partfile);
	}
// Create session GUID
	CreateGUID(&global.sessionGUID);

}

GnutellaWorker::~GnutellaWorker()
{
	char tempbuf[MAX_TEMPBUF_LEN] ;
// Let the host resolving thread end...
	WaitForSingleObject(resolveHostsAbort, INFINITE);
	Sleep(0);
	WaitForSingleObject(resolveHostsAbort, INFINITE);
// Write data
	strcpy(tempbuf, global.options.programDir) ;
	strcat(tempbuf, "SearchResults.dat") ;
	FILE *foundfile = fopen(tempbuf, "wb");
	if (foundfile)
	{
		foundFiles.Serialize(foundfile, TRUE);
		fclose(foundfile);
	}

	strcpy(tempbuf, global.options.programDir) ;
	strcat(tempbuf, "Hosts.dat") ;
	FILE *hostfile = fopen(tempbuf, "wb");
	if (hostfile)
	{
		hosts.Serialize(hostfile, TRUE);
		fclose(hostfile);
	}
// Save partial downloads
	Connection *con;
	downloadConnections.GetFirst(FALSE);
	while (con = (Connection *)downloadConnections.GetNext())
	{
		//if (con->state != GS_CONNECTED && con->state != GS_QUEUED && con->state != GS_CANCELLED && con->state != GS_FAILED && con->state != GS_DISKERROR)
		if (con->state == GS_COMPLETE || con->state == GS_NEEDOVERWRITE)
			downloadConnections.Remove(con);
		else
		{
			con->CloseSocket();
			con->state = GS_CANCELLED;
		}
	}
	strcpy(tempbuf, global.options.programDir) ;
	strcat(tempbuf, "Partials.dat") ;
	FILE *partfile = fopen(tempbuf, "wb");
	if (partfile)
	{
		downloadConnections.Serialize(partfile, TRUE);
		fclose(partfile);
	}
// Release mutexes
	CloseHandle(filesMutex);
	CloseHandle(resolveHostsMutex);
// Clean up Winsock
	WSACleanup();
}


BOOL GnutellaWorker::InitSockets()
{
	int rval;
	rval = WSAStartup(MAKEWORD(1, 1), &wsaData);
	if (rval != 0)
	{
		WSACleanup();
		return(FALSE);
	}
	if (wsaData.iMaxSockets < MAX_CONNECTIONS)
	{
		WSACleanup();
		return(FALSE);
	}
	global.localIP = MyIP();
	global.clientFirewalled = IsPrivateIP(global.localIP);

	return(TRUE);
}


LRESULT CALLBACK GnutellaWndFuncWrapper(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	return(gnutella->GnutellaWndFunc(hwnd, message, wParam, lParam));
}

BOOL GnutellaWorker::InitMessageWindow()
{
	WNDCLASS wndclass;
	wndclass.style = 0;
	wndclass.lpfnWndProc = GnutellaWndFuncWrapper;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;	 
	wndclass.hInstance = global.hInst;
	wndclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)COLOR_WINDOWFRAME;
	wndclass.lpszMenuName = NULL;
	wndclass.lpszClassName = "Crsd_Gnut_Window";

	RegisterClass(&wndclass);

	messageWindow = CreateWindow(	"Crsd_Gnut_Window",
									"If you see this, I messed up!",
									WS_CAPTION | WS_SYSMENU,
									100,
									100,
									400,
									300,
									NULL,
									NULL,
									global.hInst,
									NULL);

	return(messageWindow != NULL);
}

BOOL GnutellaWorker::InitListener()
{
// Bind listener socket
	int rval;
	serverSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (serverSocket == INVALID_SOCKET)
	{
		return(FALSE);   
	}
	struct sockaddr_in serverAddress;
	serverAddress.sin_family = AF_INET;
	serverAddress.sin_addr.s_addr = INADDR_ANY;
	serverAddress.sin_port = global.options.listenPort;
	rval = bind(serverSocket, (LPSOCKADDR)&serverAddress, sizeof(serverAddress));
	if (rval == SOCKET_ERROR)
	{
		closesocket(serverSocket);
		return(FALSE);
	}
	rval = WSAAsyncSelect(serverSocket, messageWindow, INCOMING_ACCEPT, FD_ACCEPT);
	if (rval == SOCKET_ERROR)
	{
		closesocket(serverSocket);
		return(FALSE);
	}
	rval = listen(serverSocket, 2);
	if (rval == SOCKET_ERROR)
	{
		closesocket(serverSocket);
		return(FALSE);
	}
// Start host resolving thread
	_beginthread(ProcResolveHostName, 0, NULL);
	return(TRUE);
}

BOOL GnutellaWorker::SetListenPort()
{
	int rval;
// Check for IANA assigned port numbers
	if (htons(global.options.listenPort) <= 1024)
	{
		return(FALSE);
	}

	if (serverSocket != INVALID_SOCKET)
	{
		rval = closesocket(serverSocket);
		if (rval == SOCKET_ERROR)
		{
			return(FALSE);   
		}
	}
	serverSocket = socket(AF_INET, SOCK_STREAM, 0);
	if (serverSocket == INVALID_SOCKET)
	{
		return(FALSE);   
	}
	struct sockaddr_in serverAddress;
	serverAddress.sin_family = AF_INET;
	serverAddress.sin_addr.s_addr = INADDR_ANY;
	serverAddress.sin_port = (WORD)global.options.listenPort;
	rval = bind(serverSocket, (LPSOCKADDR)&serverAddress, sizeof(serverAddress));
	if (rval == SOCKET_ERROR)
	{
		return(FALSE);
	}
	rval = WSAAsyncSelect(serverSocket, messageWindow, INCOMING_ACCEPT, FD_ACCEPT);
	if (rval == SOCKET_ERROR)
	{
		return(FALSE);
	}
	rval = listen(serverSocket, 2);
	if (rval == SOCKET_ERROR)
	{
		return(FALSE);
	}
	return(TRUE);
}

BOOL GnutellaWorker::InitFiles()
{
	int rval;
	WaitForSingleObject(filesMutex, INFINITE);
//	if (!(rval = files.fileList.Count()))
	rval = files.BuildIndex();
	ReleaseMutex(filesMutex);
	return(rval);
}

BOOL GnutellaWorker::Connect(const char *host)
{
	DWORD ip;
	WORD port;

	char *h = new char[strlen(host)+1], *startport;
	strcpy(h, host);
	if (startport = strchr(h, ':'))
	{
		*startport = '\0';
		int tempport = atoi(startport+1);
		if (!tempport)
		{
			delete [] h;
			return(FALSE);
		}
		port = htons((WORD)tempport);
	}
	else
	{
		port = htons(DEFAULT_GNUTELLAPORT);
	}
	ip = inet_addr(h);
	if (ip == INADDR_NONE)
	{
		if (WaitForSingleObject(resolveHostsMutex, 100) == WAIT_OBJECT_0)
		{
			HostInfo *hi = new HostInfo;
			hi->port = port;
			hi->hostName = h;
			hi->resolved = FALSE;
			hi->resolveType = RESOLVE_NAMETOADDRESS;
			resolveHosts.Add(hi->resolveType, hi);
			ReleaseMutex(resolveHostsMutex);
		}
		delete [] h;
		return(TRUE);
	}
	delete [] h;
	if (!Connect(ip, port))
	{
		return(FALSE);
	}
	return(TRUE);
}

BOOL GnutellaWorker::Connect(DWORD ip, WORD port)
{
	int rval;

	if (ip == global.localIP)
		return(FALSE);
	if (IsPrivateIP(ip))
		return(FALSE);

	Connection *con = new Connection;
	con->type = CT_OUTGOING;
	con->id = ++global.connectionID;
	con->socket = socket(PF_INET, SOCK_STREAM, 0);
	con->state = GS_WAITINGCONNECT;
	if (con->socket == INVALID_SOCKET)
	{
		delete con;
		return(FALSE);
	}
	con->address.sin_family = AF_INET;
	con->conIP = ip;
	con->conPort = port;
	char host[MAX_TEMPBUF_LEN];
	char *thost = inet_ntoa(con->address.sin_addr);
	strcpy(host, thost);

// Check whether we already have a connection to that address
	Connection *dummy;
	gnutellanetConnections.GetFirst(FALSE);
	for (int i=0; dummy = (Connection *)gnutellanetConnections.GetNext(); i++)
	{
		if (con->conIP == dummy->conIP &&
			con->conPort == dummy->conPort &&
			con->type == con->type)
		{
			delete con;
			return(TRUE);
		}
	}

	WSAAsyncSelect(con->socket, messageWindow, GNUTELLANET_CONNECTED, FD_CONNECT);
	rval = connect(con->socket, (const struct sockaddr *)&con->address, sizeof(struct sockaddr_in));
	if (rval == SOCKET_ERROR && WSAGetLastError() == WSAEWOULDBLOCK)
	{
	}
	else if (rval == SOCKET_ERROR)
	{
		delete con;
		return(FALSE);
	}

	con->lastTime = GetTickCount();        // Update time of last action ..
	rval = gnutellanetConnections.Add(con->id, con);
	if (!rval) return FALSE ;

	SetEvent(gnutellanetEvent);
	
	return(TRUE);
}

BOOL GnutellaWorker::Search(const char *requestedsearch)
{
	GnutellaHeader gh;
	GnutellaQuery gq;
	GLUID guid;

	char search[MAX_PATH] ;
	
	strcpy(search, requestedsearch) ;
	// strcat(search, " .mp3") ;

	if (!CreateGUID(&guid))
		return(FALSE);
	memcpy(gh.guid, &guid, 16);
	memcpy(&currentSearchGUID, &guid, 16);
	gh.function = GNUTELLA_SEARCH;
	gh.ttl = (BYTE)global.options.searchTTL;
	gh.hops = 0;
	gq.minspeed = 0;
	int tosend = sizeof(GnutellaHeader) + sizeof(GnutellaQuery) + strlen(search) + 1;
	char *senddata = new char[tosend];
	gh.length = sizeof(GnutellaQuery) + strlen(search) + 1;
	memcpy(senddata, &gh, sizeof(GnutellaHeader));
	memcpy(senddata+sizeof(GnutellaHeader), &gq, sizeof(GnutellaQuery));
	strcpy(senddata+sizeof(GnutellaHeader)+sizeof(GnutellaQuery), search);

	foundFiles.Reset();

	Logger::Instance()->Log("Search: %s -+- \n", requestedsearch);
	Logger::Instance()->LogGnutellaHeader(gh);
	Logger::Instance()->LogGnutellaQuery(gq);

	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	for (int i=0; con = (Connection *)gnutellanetConnections.GetNext(); i++)
	{
		unsigned char ip[4];

		if (con->GetIp(ip))
			Logger::Instance()->Log("Sent query to IP %u.%u.%u.%u\n", 
				(unsigned int)(ip[0]),(unsigned int)(ip[1]),
				(unsigned int)(ip[2]),(unsigned int)(ip[3]));
		if (con->state != GS_CONNECTED)
			continue;
		con->SendPacket(senddata, tosend);
	}
	delete [] senddata;

	return(TRUE);
}

BOOL GnutellaWorker::GetFile(FileInfo fi, BOOL overwrite)
{
// Check whether the file already exists 
	String savename;
	savename = global.options.downloadDir;
	savename.Concat(fi.name);
	FILE *tempfile = fopen(savename, "rb");
// Either resume, overwrite it or return
	Connection *con = new Connection;
	con->fi = fi;
	con->fi.directory = global.options.downloadDir;
	con->type = CT_GET;
	con->state = GS_QUEUED;
	con->id = ++global.connectionID;
	con->address.sin_family = AF_INET;
	con->conIP = fi.hostIP;
	con->conPort = htons(fi.hostPort);
	if (tempfile != NULL)
	{
		if (overwrite)
		{
			fclose(tempfile);
			tempfile = fopen(savename, "wb");
			if (!tempfile)
			{
				con->state = GS_DISKERROR;
			}
			else
				fclose(tempfile);
		}
		else
		{
			con->transmitted = filelength(fileno(tempfile));
			fclose(tempfile);
			if (con->transmitted >= fi.size)
			{
				con->state = GS_NEEDOVERWRITE;
			}
		}
	}

	downloadConnections.Add(con->id, con);
	SetEvent(getEvent);

	return(TRUE);
}

BOOL GnutellaWorker::StartGet(Connection *con)
{
// Check if the host is behind a firewall, if yes, send a push
	if (IsPrivateIP(con->conIP))
	{
		if (IsPrivateIP(global.localIP))
		{
			con->state = GS_FAILED;
			return(TRUE);
		}
		GnutellaHeader gh;
		gh.function = GNUTELLA_PUSHREQUEST;
		memcpy(gh.guid, con->fi.guid, sizeof(GLUID));
		gh.hops = 0;
		gh.ttl = global.options.searchTTL;
		gh.length = sizeof(GnutellaPushRequest);
		GnutellaPushRequest gpr;
		memcpy(gpr.guid, con->fi.clientGUID, sizeof(GLUID));
		gpr.index = con->fi.index;
		gpr.ip = global.localIP;
		gpr.port = ntohs(global.options.listenPort);
	// Flood the message for the first step
		int msglen = sizeof(GnutellaHeader) + sizeof(GnutellaPushRequest);
		char *msg = new char[msglen];
		memcpy(msg, &gh, sizeof(GnutellaHeader));
		memcpy(msg+sizeof(GnutellaHeader), &gpr, sizeof(GnutellaPushRequest));
		Connection *dummy;
		gnutellanetConnections.GetFirst(FALSE);
		while (dummy = (Connection *)gnutellanetConnections.GetNext())
		{
			if (dummy->state != GS_CONNECTED)
				continue;
			dummy->SendPacket(msg, msglen);
		}
		delete [] msg;

		FileInfo *fi2;
		fi2 = new FileInfo(con->fi);
		requestedPushes.Add(fi2);
		con->state = GS_WAITINGPUSH;

		SetEvent(getEvent);

		return(TRUE);
	}

	con->socket = socket(PF_INET, SOCK_STREAM, 0);
	if (con->socket == INVALID_SOCKET)
	{
		delete con;
		return(FALSE);
	}
	con->state = GS_WAITINGCONNECT;
	WSAAsyncSelect(con->socket, messageWindow, GET_CONNECTED, FD_CONNECT);
	int rval = connect(con->socket, (const struct sockaddr *)&con->address, sizeof(struct sockaddr_in));
	if (rval == SOCKET_ERROR && WSAGetLastError() == WSAEWOULDBLOCK)
	{
	}
	else if (rval == SOCKET_ERROR)
	{
		delete con;
		return(FALSE);
	}

	SetEvent(getEvent);

	return(TRUE);
}

BOOL GnutellaWorker::RemoveDownloadConnection(int id)
{
	Connection *con = (Connection *)downloadConnections.GetData(id);
	if (con)
	{
		downloadConnections.Remove(con);
		return(TRUE);
	}
	return(FALSE);
}

BOOL GnutellaWorker::RemoveUploadConnection(int id)
{
	Connection *con = (Connection *)uploadConnections.GetData(id);
	if (con)
	{
		uploadConnections.Remove(con);
		return(TRUE);
	}
	return(FALSE);
}

BOOL GnutellaWorker::RequestPush(FileInfo fi)
{
// Set up the push request
	GnutellaHeader gh;
	gh.function = GNUTELLA_PUSHREQUEST;
	CreateGUID(&gh.guid);
	gh.hops = 0;
	gh.ttl = global.options.searchTTL;
	gh.length = sizeof(GnutellaPushRequest);
	GnutellaPushRequest gpr;
	memcpy(gpr.guid, global.sessionGUID, sizeof(GLUID));
	gpr.index = fi.index;
	gpr.ip = global.localIP;
	gpr.port = global.options.listenPort;
// Flood the message for the first step
	int msglen = sizeof(GnutellaHeader) + sizeof(GnutellaPushRequest);
	char *msg = new char[msglen];
	memcpy(msg, &gh, sizeof(GnutellaHeader));
	memcpy(msg+sizeof(GnutellaHeader), &gpr, sizeof(GnutellaPushRequest));
	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	for (int i=0; con = (Connection *)gnutellanetConnections.GetNext(); i++)
	{
		if (con->state != GS_CONNECTED)
			continue;
		con->SendPacket(msg, msglen);
	}
	delete [] msg;

	FileInfo *fi2;
	fi2 = new FileInfo(fi);
	requestedPushes.Add(fi2);

	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::HandleConnect(Connection *sourcecon, const char *buf, int buflen)
{
	sourcecon->lastTime = GetTickCount();

	if (memcmp(buf, "GNUTELLA CONNECT/0.4", strlen("GNUTELLA CONNECT/0.4")))
		return(FALSE);

	sourcecon->SendPacket("GNUTELLA OK\n\n", strlen("GNUTELLA OK\n\n"));
	sourcecon->state = GS_CONNECTED;
	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::HandleConnectResponse(Connection *sourcecon, const char *buf, int buflen)
{
	int rval;

	rval = memcmp(buf, "GNUTELLA OK\n\n", strlen("GNUTELLA OK\n\n"));
	if (rval)
		return(FALSE);

	sourcecon->lastTime = GetTickCount();
	sourcecon->state = GS_CONNECTED;

	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::HandleInit(Connection *sourcecon, const char *buf, int buflen)
{
// Check if we already got that message
	GnutellaHeader gh;
	memcpy(&gh, buf, sizeof(GnutellaHeader));
	RoutingInfo *ritemp;
	int guidDesc;
	memcpy(&guidDesc, gh.guid, sizeof(int));
	pingRoutes.GetFirst(FALSE);
	for (int i=0; ritemp = (RoutingInfo *)pingRoutes.GetNextByDescriptor(guidDesc); i++)
	{
		if (!memcmp(ritemp->guid, gh.guid, sizeof(GLUID)))
			return(TRUE);
	}
	global.stats.pings++;
// If it's a temporary connection, return some cached hosts
	GnutellaHeader gh2;
	GnutellaInitResponse gir;
	int tosend = sizeof(GnutellaHeader)+sizeof(GnutellaInitResponse), rval;
	char *senddata;
	if (sourcecon->temporary)
	{
		memcpy(gh2.guid, gh.guid, 16);
		gh2.function = GNUTELLA_INITRESPONSE;
		gh2.ttl = (BYTE)(gh.hops+1);
		gh2.hops = 0;
		gh2.length = sizeof(GnutellaInitResponse);
		int tosend = sizeof(GnutellaHeader)+sizeof(GnutellaInitResponse);
		senddata = new char[tosend];
		int i=0;
		HostInfo *hi;
		do
		{
			hi = (HostInfo *)hosts.GetRandom();
			if (!hi)
				break;
			gir.hostIP = hi->ip;
			gir.hostPort = ntohs(hi->port);
			gir.fileCount = hi->fileCount;
			gir.filesTotalsize = hi->filesTotalsize;
			memcpy(senddata, (BYTE *)&gh2, sizeof(GnutellaHeader));
			memcpy(senddata+sizeof(GnutellaHeader), (BYTE *)&gir, sizeof(GnutellaInitResponse));
			rval = sourcecon->SendPacket(senddata, tosend);
			i++;
		}
		while (i < 50 && i < hosts.Count() && rval != SOCKET_ERROR);
		delete [] senddata;
		SetEvent(gnutellanetEvent);
		return(TRUE);
	}
// Don't send a pong if we won't accept connections
	if (global.options.acceptIncoming && !global.clientFirewalled)
	{
	// Parse the request and handle it locally
		memcpy(gh2.guid, gh.guid, 16);
		gh2.function = GNUTELLA_INITRESPONSE;
		gh2.ttl = (BYTE)(gh.hops+1);
		gh2.hops = 0;
		gh2.length = sizeof(GnutellaInitResponse);
		if (global.options.forcedIP)
			gir.hostIP = global.options.forcedIP;
		else
			gir.hostIP = global.localIP;
		gir.hostPort = ntohs(global.options.listenPort);
		if (WaitForSingleObject(filesMutex, 0) == WAIT_OBJECT_0)
		{
			gir.fileCount = files.fileList.Count();
			gir.filesTotalsize = files.TotalSize();
			ReleaseMutex(filesMutex);
		}
	// Send a reply
		tosend = sizeof(GnutellaHeader)+sizeof(GnutellaInitResponse);
		senddata = new char[tosend];
		memcpy(senddata, (BYTE *)&gh2, sizeof(GnutellaHeader));
		memcpy(senddata+sizeof(GnutellaHeader), (BYTE *)&gir, sizeof(GnutellaInitResponse));
		sourcecon->SendPacket(senddata, tosend);
		delete [] senddata;
	}
// Store the routing information
	RoutingInfo *ri;
	ri = new RoutingInfo;
	ri->function = GNUTELLA_INIT;
	ri->time = GetTickCount();
	memcpy(ri->guid, gh.guid, sizeof(GLUID));
	ri->id = sourcecon->id;
	memcpy(&guidDesc, ri->guid, sizeof(int));
	pingRoutes.Add(guidDesc, ri);
// And pass the message
	char *newmsg = new char[buflen];
	memcpy(newmsg, buf, buflen);
	GnutellaHeader *ngh;
	ngh = (GnutellaHeader *)newmsg;
	ngh->hops++;
	if (ngh->hops > global.options.pingTTL)
	{
		delete [] newmsg;
		return(TRUE);
	}
	ngh->ttl--;
	if (ngh->ttl <= 0)
	{
		delete [] newmsg;
		return(TRUE);
	}
	if (ngh->ttl > global.options.pingTTL)
		ngh->ttl = global.options.pingTTL;
	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	for (i=0; con = (Connection *)gnutellanetConnections.GetNext(); i++)
	{
		if (con->state != GS_CONNECTED)
			continue;
		if (con == sourcecon)
			continue;
		con->SendPacket(newmsg, buflen);
	}
	delete [] newmsg;
// Notify the GUI
	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::HandleInitResponse(Connection *sourcecon, const char *buf, int buflen)
{
// If this is only a temporary connection, disconnect it
	if (sourcecon->temporary)
		return(FALSE);
	global.stats.pongs++;
// Parse the message and update the NetStats
	GnutellaHeader gh;
	GnutellaInitResponse gir;
	memcpy(&gh, buf, sizeof(GnutellaHeader));
	memcpy(&gir, buf+sizeof(GnutellaHeader), sizeof(GnutellaInitResponse));
// Add the host to the host list if it's probably no duplicate
	HostInfo *hi = new HostInfo;
	hi->ip = gir.hostIP;
	hi->port = htons(gir.hostPort);
	hi->time = GetTickCount();
	hi->fileCount = gir.fileCount;
	hi->filesTotalsize = gir.filesTotalsize;
	HostInfo *dummy;
	hosts.GetFirst(FALSE);
	while (dummy = (HostInfo *)hosts.GetNextByDescriptor(hi->ip))
	{
		if (hi->port == dummy->port)
		{
			global.totalFileCount -= dummy->fileCount;
			global.totalFileSize -= dummy->filesTotalsize/100;
			hosts.Remove(dummy) ;
		}
	}
	if (hi)
	{
		hosts.Add(hi->ip, hi);
		global.totalFileCount += hi->fileCount;
		global.totalFileSize += hi->filesTotalsize/100;
	}
// Check whether we have to route the pong
	int guidDesc;
	memcpy(&guidDesc, gh.guid, sizeof(int));
	pingRoutes.GetFirst(FALSE);
	RoutingInfo *srcri;
	for (int i=0; srcri = (RoutingInfo *)pingRoutes.GetNextByDescriptor(guidDesc); i++)
	{
		if (!memcmp(srcri->guid, gh.guid, sizeof(GLUID)))
			break;
	}
	if (srcri)
	{
		Connection *con = (Connection *)gnutellanetConnections.GetData(srcri->id);
		if (con)
		{
		// Everything went fine, so pass the response along
			char *newmsg = new char[buflen];
			memcpy(newmsg, buf, buflen);
			GnutellaHeader *ngh;
			ngh = (GnutellaHeader *)newmsg;
			ngh->hops++;
			ngh->ttl--;
			if (ngh->hops < global.options.pingTTL && ngh->ttl > 0)
			{
				if (ngh->ttl > global.options.searchTTL)
					ngh->ttl = global.options.searchTTL;
				if (con->SendPacket(newmsg, buflen) == SOCKET_ERROR)
				{
					Packet *packet = new Packet;
					packet->data.Alloc(buflen, newmsg);
					packet->time = GetTickCount();
					con->packets.Add(packet);
				}
			}
			delete [] newmsg;
		}
	}
// Notify the GUI
	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::HandlePush(Connection *con, const char *buf, int buflen)
{
	Logger::Instance()->Log("HandlePush\n");
// Parse the GIV message
	if (memcmp(buf+buflen-2, "\n\n", strlen("\n\n")))
		return(FALSE);
	String str(buf, buflen-2);
	String filename = str.RightOfLast('/');
	String serverguid = str.LeftOfLast('/');
	String sindex = serverguid.LeftOf(':');
	sindex = sindex.RightOf(' ');
	serverguid = serverguid.RightOf(':');
	int index = atoi(sindex);
	GLUID guid;
	char c;
	for (int i=0, j=0; i<16; i++)
	{
		c = serverguid[j++];
		if (c >= '0' && c <= '9')
			guid[i] = (c - '0') * 16;
		else if (c >= 'A' && c <= 'F')
			guid[i] = (c - 'A' + 10) * 16;

		c = serverguid[j++];
		if (c >= '0' && c <= '9')
			guid[i] += c - '0';
		else if (c >= 'A' && c <= 'F')
			guid[i] += c - 'A' + 10;
	}
// Check if we actually requested this push
	FileInfo *dfi;
	requestedPushes.GetFirst(FALSE);
	while (dfi = (FileInfo *)requestedPushes.GetNext())
	{
		if (!memcmp(dfi->clientGUID, guid, sizeof(GLUID)) && dfi->index == index && dfi->name == filename)
			break;
	}
	if (!dfi)
		return(FALSE);
// If yes, send the HTTP request
	con->fi = *dfi;
	con->type = CT_GET;
	HTTPHeader httph;
	httph.isResponse = FALSE;
	char tempstring[MAX_TEMPBUF_LEN];
	sprintf(tempstring, "/get/%d/%s", con->fi.index, (LPCSTR)con->fi.name);
	httph.fileName = tempstring;
	httph.type = "GET";
	httph.versionString = "HTTP/1.0";
	httph.userAgent = "Gnutella";
	if (con->transmitted)
	{
		sprintf(tempstring, "bytes=%d-", con->transmitted);
		httph.range = tempstring;
	}
	httph.Build();
	int rval = con->SendPacket((LPCSTR)httph.header, httph.header.GetLength());
	if (rval == SOCKET_ERROR)
	{
		downloadConnections.Remove(con);
		return(FALSE);
	}
	gnutellanetConnections.Unlink(con);
	Connection *dummy;
	downloadConnections.GetFirst(FALSE);
	while (dummy = (Connection *)downloadConnections.GetNext())
	{
		if (dummy->state == GS_WAITINGPUSH && dummy->fi.name == con->fi.name)
		{
			// Yes, I know this isn't 100% safe, but who cares...
			downloadConnections.Remove(dummy);
			break;
		}
	}
	downloadConnections.Add(con->id, con);
	rval = WSAAsyncSelect(con->socket, messageWindow, GET_READ, FD_READ | FD_CLOSE);
	
	return(TRUE);
}

BOOL GnutellaWorker::HandlePushRequest(Connection *sourcecon, const char *buf, int buflen)
{
	Logger::Instance()->Log("HandlePushRequest\n");
// If this is only a temporary connection, disconnect it
	if (sourcecon->temporary)
		return(FALSE);
// Is the push request for us?
	GnutellaHeader gh;
	GnutellaPushRequest gpr;
	memcpy(&gh, buf, sizeof(GnutellaHeader));
	memcpy(&gpr, buf+sizeof(GnutellaHeader), sizeof(GnutellaPushRequest));
	if (!memcmp(global.sessionGUID, gpr.guid, sizeof(GLUID)))
	{
		PushFile(sourcecon, gpr);
		return(TRUE);
	}
// Do we know who the message is for?
	RoutingInfo *ritemp;
	int guidDesc;
	memcpy(&guidDesc, gpr.guid, sizeof(int));
	pushRoutes.GetFirst(FALSE);
	for (int i=0; ritemp = (RoutingInfo *)pushRoutes.GetNextByDescriptor(guidDesc); i++)
	{
		if (!memcmp(ritemp->guid, gpr.guid, sizeof(GLUID)))
			break;
	}
	if (!ritemp)
		return(TRUE);
// Yes, so try to pass along
	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	con = (Connection *)gnutellanetConnections.GetData(ritemp->id);
	if (con)
	{
	// Everything went fine, so pass the responseCode along
		char *newmsg = new char[buflen];
		memcpy(newmsg, buf, buflen);
		GnutellaHeader *ngh;
		ngh = (GnutellaHeader *)newmsg;
		ngh->hops++; ngh->ttl--;
		if (ngh->hops < global.options.searchTTL && ngh->ttl > 0)
		{
			if (ngh->ttl > global.options.searchTTL)
				ngh->ttl = global.options.searchTTL;
			if (con->SendPacket(newmsg, buflen) == SOCKET_ERROR)
			{
				Packet *packet = new Packet;
				packet->data.Alloc(buflen, newmsg);
				packet->time = GetTickCount();
				con->packets.Add(packet);
			}
		}
		delete [] newmsg;
	}
// Notify the GUI
	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::PushFile(Connection *oldcon, GnutellaPushRequest gpr)
{
	if (WaitForSingleObject(filesMutex, 0) == WAIT_TIMEOUT) // FIXME
		return(FALSE);

	FileInfo *fi = (FileInfo *)files.fileList.GetData(gpr.index);
	if (!fi)
	{
		ReleaseMutex(filesMutex);
		return(TRUE);
	}

	Connection *con = new Connection;
	con->fi = *fi;
	memcpy(con->fi.clientGUID, gpr.guid, sizeof(GLUID));
	con->type = CT_SEND;
	con->state = GS_WAITINGCONNECT;
	con->id = ++global.connectionID;
	con->socket = ::socket(PF_INET, SOCK_STREAM, 0);
	if (con->socket == INVALID_SOCKET)
	{
		delete con;
		ReleaseMutex(filesMutex);
		return(FALSE);
	}
	con->address.sin_family = AF_INET;
	con->conIP = gpr.ip;
	con->conPort = htons(gpr.port);
	con->transmitted = 0;

	con->type = CT_SEND;
	con->id = ++global.connectionID;
	con->socket = socket(PF_INET, SOCK_STREAM, 0);
	if (con->socket == INVALID_SOCKET)
	{
		delete con;
		ReleaseMutex(filesMutex);
		return(FALSE);
	}
	con->address.sin_family = AF_INET;
	WSAAsyncSelect(con->socket, messageWindow, PUSH_CONNECTED, FD_CONNECT);
	int rval = connect(con->socket, (const struct sockaddr *)&con->address, sizeof(struct sockaddr_in));
	if (rval == SOCKET_ERROR && WSAGetLastError() == WSAEWOULDBLOCK)
	{
	}
	else if (rval == SOCKET_ERROR)
	{
		delete con;
		ReleaseMutex(filesMutex);
		return(FALSE);
	}
	gnutellanetConnections.Add(con->id, con);

	SetEvent(sendEvent);
	ReleaseMutex(filesMutex);

	return(TRUE);
}

BOOL GnutellaWorker::GetTime(int index)
{
	const char *timeServers[] = {
		"132.163.4.101", // time-a.timefreq.bldrdoc.gov
		"132.163.4.102",
		"132.163.4.103"
	};
	const int nServers = sizeof(timeServers) / sizeof(char *);


	Connection *con = new Connection;
	con->type = CT_TIME;
	con->state = GS_WAITINGCONNECT;
	con->id = ++global.connectionID;
	con->socket = ::socket(PF_INET, SOCK_STREAM, 0);
	if (con->socket == INVALID_SOCKET)
	{
		delete con;
		return(FALSE);
	}
	con->address.sin_family = AF_INET;
	con->conIP = inet_addr(timeServers[index % nServers ]);  
	con->conPort = htons(37);  // TIME protocol port
	con->transmitted = 0;
	WSAAsyncSelect(con->socket, messageWindow, GETTIME_CONNECTED, FD_CONNECT);
	int rval = connect(con->socket, (const struct sockaddr *)&con->address, sizeof(struct sockaddr_in));
	if (rval == SOCKET_ERROR && WSAGetLastError() == WSAEWOULDBLOCK)
	{
	}
	else if (rval == SOCKET_ERROR)
	{
		delete con;
		return(FALSE);
	}
	gnutellanetConnections.Add(con->id, con);
	return(TRUE);
}

BOOL GnutellaWorker::HandleSearch(Connection *sourcecon, const char *buf, int buflen)
{
	Logger::Instance()->Log("HandleSearch:\n");
// If this is only a temporary connection, disconnect it
	if (sourcecon->temporary)
		return(FALSE); // Some bug took me 15 hours here to fix. RIP hehehehehe
// Check if we already got that message
	GnutellaHeader gh;
	memcpy(&gh, buf, sizeof(GnutellaHeader));
	Logger::Instance()->LogGnutellaHeader(gh);
	RoutingInfo *ritemp;
	int guidDesc;
	memcpy(&guidDesc, gh.guid, sizeof(int));
	searchRoutes.GetFirst(FALSE);
	for (int i=0; ritemp = (RoutingInfo *)searchRoutes.GetNextByDescriptor(guidDesc); i++)
	{
		if (!memcmp(ritemp->guid, gh.guid, sizeof(GLUID)))
		{
			Logger::Instance()->Log("guid found: exit\n");
			return(TRUE);
		}
	}
	Logger::Instance()->Log("guid not found: performing search\n");
	global.stats.searches++;
// No, then parse it
	GnutellaQuery gq;
	GnutellaQueryResponseHeader gqrh;
	GnutellaQueryResponseEntry gqre;
	memcpy(&gq, buf+sizeof(GnutellaHeader), sizeof(GnutellaQuery));
// Extract the query string
	String query(buf+sizeof(GnutellaHeader)+sizeof(GnutellaQuery));
// Do a file search only if we're fast enough
	query.ToLower();
	if (gq.minspeed <= global.options.connectionSpeed && query.Find("mpg") == -1 && query.Find("asf") == -1 && query.Find("xxx") == -1 && query.Find("sex") == -1 && query.Find("fuck") == -1 && query.Find("porn") == -1) 
	{
	// Search the file database
		Logger::Instance()->Log("search the file database for %s\n", (LPCSTR)query);
		List FoundList;
		int found;
		if (WaitForSingleObject(filesMutex, 0) == WAIT_OBJECT_0)
		{
			found = files.SearchFile(query, &FoundList);
			ReleaseMutex(filesMutex);
		}
		else
			found = 0;
	// Send an answer
		if (found)
		{
			Logger::Instance()->Log("files found\n");
			char *senddata = new char[60000];
			if (global.options.forcedIP)
				gqrh.ip = global.options.forcedIP;
			else
				gqrh.ip = global.localIP;

			unsigned char ip[4];
			memcpy(ip, &gqrh.ip, 4);
			Logger::Instance()->Log("My IP %u.%u.%u.%u\n", 
				(unsigned int)(ip[0]),(unsigned int)(ip[1]),
				(unsigned int)(ip[2]),(unsigned int)(ip[3]));
			gqrh.port = ntohs(global.options.listenPort);
			gqrh.speed = global.options.connectionSpeed;
			int tosend = sizeof(GnutellaHeader) + sizeof(GnutellaQueryResponseHeader);
			FoundList.GetFirst(FALSE);
			found = 0;
			FileInfo *fi;
			for (i=0; (fi = (FileInfo *)FoundList.GetNext()) && (i < global.options.searchLimit); i++)
			{
				gqre.index = fi->index;
				gqre.size = fi->size;
				memcpy(senddata + tosend, &gqre, sizeof(GnutellaQueryResponseEntry));
				tosend += sizeof(GnutellaQueryResponseEntry);
				memcpy(senddata + tosend, fi->name, fi->name.GetLength());
				tosend += fi->name.GetLength();
				memcpy(senddata + tosend, "\0\0", 2);
				tosend += 2;
				found++;
				if (tosend > 59700)
					break;
			}
			gqrh.numEntries = (BYTE)found;
			memcpy(senddata + tosend, &global.sessionGUID, 16);
			tosend += 16;
			gh.function = GNUTELLA_SEARCHRESPONSE;
			gh.ttl = (BYTE)(gh.hops+2);
			gh.hops = 0;
			gh.length = tosend - sizeof(GnutellaHeader);
			memcpy(senddata, &gh, sizeof(GnutellaHeader));
			memcpy(senddata+sizeof(GnutellaHeader), &gqrh, sizeof(GnutellaQueryResponseHeader));
			if (sourcecon->SendPacket(senddata, tosend) == SOCKET_ERROR)
			{
				Packet *packet = new Packet;
				packet->data.Alloc(tosend, senddata);
				packet->time = GetTickCount();
				sourcecon->packets.Add(packet);
			}
			delete [] senddata;
		}
	}
// Store the routing information
	RoutingInfo *ri;
	ri = new RoutingInfo;
	ri->function = GNUTELLA_SEARCH;
	ri->time = GetTickCount();
	memcpy(ri->guid, gh.guid, sizeof(GLUID));
	//fyhash((BYTE *)buf + sizeof(GnutellaHeader), buflen - sizeof(GnutellaHeader), ri->hash);
	ri->id = sourcecon->id;
	memcpy(&guidDesc, ri->guid, sizeof(int));
	searchRoutes.Add(guidDesc, ri);
// And pass the message
	char *newmsg = new char[buflen];
	memcpy(newmsg, buf, buflen);
	GnutellaHeader *ngh;
	ngh = (GnutellaHeader *)newmsg;
	ngh->hops++;
	ngh->ttl--;
	if (ngh->hops <= global.options.searchTTL && ngh->ttl > 0)
	{
		if (ngh->ttl > global.options.searchTTL)
			ngh->ttl = global.options.searchTTL;
		Connection *con;
		gnutellanetConnections.GetFirst(FALSE);
		for (i=0; con = (Connection *)gnutellanetConnections.GetNext(); i++)
		{
			if (con->state != GS_CONNECTED)
				continue;
			if (con == sourcecon)
				continue;
			unsigned char ip[4];

			if (con->GetIp(ip))
				Logger::Instance()->Log("Packet forwarded to IP %u.%u.%u.%u\n", 
					(unsigned int)(ip[0]),(unsigned int)(ip[1]),
					(unsigned int)(ip[2]),(unsigned int)(ip[3]));
			con->SendPacket(newmsg, buflen);
		}
	}
	delete [] newmsg;
// Notify the GUI
	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::HandleSearchResponse(Connection *sourcecon, const char *buf, int buflen)
{
	Logger::Instance()->Log("HandleSearchResponse\n");

	int guidDesc;
	int i ;
	RoutingInfo *ri, *ri2 ;
	char filename[MAX_TEMPBUF_LEN];
	GnutellaHeader gh;
	GnutellaQueryResponseHeader gqrh;
	GnutellaQueryResponseEntry gqre;

// If this is only a temporary connection, disconnect it
	if (sourcecon->temporary) 
		return(FALSE);
	global.stats.searchresponses++;

// Parse the search
	ri = ri2 = new RoutingInfo ;
	memcpy(&gh, buf, sizeof(GnutellaHeader));

	Logger::Instance()->LogGnutellaHeader(gh);
	ri->function = GNUTELLA_SEARCHRESPONSE;
	ri->time = GetTickCount();
	ri->id = sourcecon->id;
	memcpy(ri->guid, buf+buflen-16, sizeof(GLUID));
	memcpy(&guidDesc, ri->guid, sizeof(int));

	if (memcmp(gh.guid, &currentSearchGUID, 16))
	{
		Logger::Instance()->Log("this response is not the current search GUID\n");
		pushRoutes.Add(guidDesc, ri) ;
		gnutellanetConnections.GetFirst(FALSE);
		RoutingInfo *srcri = NULL;
		searchRoutes.GetFirst(FALSE);
		for (i=0; srcri = (RoutingInfo *)searchRoutes.GetNextByDescriptor(guidDesc); i++)
		{
			if (!memcmp(srcri->guid, gh.guid, sizeof(GLUID))) 
				break;
		}
		if (srcri)
		{
			Logger::Instance()->Log("this response is not the current search GUID but I found it in the routing table\n");
			Connection *con = NULL;
			con = (Connection *)gnutellanetConnections.GetData(srcri->id);
			if (con)
			{
			// Everything went fine, so pass the response along
				char *newmsg = new char[buflen];
				memcpy(newmsg, buf, buflen);
				GnutellaHeader *ngh;
				ngh = (GnutellaHeader *)newmsg;
				ngh->hops++;
				ngh->ttl--;
				if (ngh->hops < global.options.pingTTL && ngh->ttl > 0)
				{
					if (ngh->ttl > global.options.searchTTL)
						ngh->ttl = global.options.searchTTL;
					if (con->SendPacket(newmsg, buflen) == SOCKET_ERROR)
					{
						Packet *packet = new Packet;
						packet->data.Alloc(buflen, newmsg);
						packet->time = GetTickCount();
						con->packets.Add(packet);
					}
				}
				delete [] newmsg;
			}
		}

		if (global.options.displayAll) 
		{
			ri = new RoutingInfo ;
			memcpy(ri, ri2, sizeof(RoutingInfo)) ;
		}
		else return TRUE ;
	}

// It's a new one, so add the file names to foundFiles
	Logger::Instance()->Log("new response, I add it to the found files\n");
	memcpy(&gqrh, buf+sizeof(GnutellaHeader), sizeof(GnutellaQueryResponseHeader));
	const char *msgiter = buf + sizeof(GnutellaHeader) + sizeof(GnutellaQueryResponseHeader);
	FileInfo *fi;
	String fn, ignore, allIgnores;
	for (i=0; i < gqrh.numEntries; i++)
	{
		memcpy(&gqre, msgiter, sizeof(GnutellaQueryResponseEntry));
		msgiter += sizeof(GnutellaQueryResponseEntry);
		strcpy(filename, msgiter);
		msgiter += strlen(filename) + 2;
		fn = filename;
		allIgnores = global.options.ignoreResults;
		fn.ToLower() ;
		allIgnores.ToLower() ;
		while (allIgnores.GetLength())
		{
			ignore = allIgnores.LeftOf(';');
			if (fn.MatchWildcard(ignore)) break;
			allIgnores = allIgnores.RightOf(';');
		}

		if (!allIgnores.GetLength())
		{
			fi = new FileInfo;
			fi->name = filename;
			fi->size = gqre.size;
			fi->index = gqre.index;
			fi->hostSpeed = gqrh.speed;
			fi->hostIP = gqrh.ip;
			fi->hostPort = gqrh.port;
			GetLocalTime(&fi->SystemTime) ;
			memcpy(fi->guid, gh.guid, sizeof(GLUID));
			memcpy(fi->clientGUID, ri->guid, sizeof(GLUID));
			foundFiles.Add(global.searchFileCount, fi);
			global.searchFileCount++ ;
		}
	}

	Logger::Instance()->Log("Adding new receivedResults %x%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x\n",
		ri->guid[0],
		ri->guid[1],
		ri->guid[2],
		ri->guid[3],
		ri->guid[4],
		ri->guid[5],
		ri->guid[6],
		ri->guid[7],
		ri->guid[8],
		ri->guid[9],
		ri->guid[10],
		ri->guid[11],
		ri->guid[12],
		ri->guid[13],
		ri->guid[14],
		ri->guid[15]
		);
	receivedResults.Add(guidDesc, ri) ;
	PostMessage(global.hWnd, CHANGED_FOUND_FILES, 1, 0);
	return(TRUE);

}





BOOL GnutellaWorker::HandleGetRequest(Connection *sourcecon, const char *buf, int buflen)
{
	Logger::Instance()->Log("HandleGetRequest\n");
	if (WaitForSingleObject(filesMutex, 0) == WAIT_TIMEOUT)
		return(FALSE);
// Check whether sending the file would break any upload limits
	int uploading;
	uploadConnections.GetFirst(FALSE);
	// Total uploads
	uploading = 0;
	Connection *dummy;
	HTTPHeader httph;
	httph.isResponse = FALSE;
	httph.header = String(buf, buflen);
	httph.Parse();
	String sname = httph.fileName.RightOfLast('/');
	while (dummy = (Connection *)uploadConnections.GetNext())
	{
		if (dummy->socket == INVALID_SOCKET)
			continue;
		uploading++;
	}
	if (uploading >= global.options.maxUploadstotal)
	{
		Logger::Instance()->Log("error 503 because greater than maxUploadstotal\n");
		sourcecon->responseCode = RC_503;
	}
	if (sourcecon->responseCode != RC_503)
	{
		// Uploads per host
		uploading = 0;
		uploadConnections.GetFirst(FALSE);
		while (dummy = (Connection *)uploadConnections.GetNext())
		{
			if (dummy->socket == INVALID_SOCKET)
				continue;
			if (sourcecon->conIP == dummy->conIP && sourcecon->conPort == dummy->conPort)
				uploading++;
		}
		if (uploading >= global.options.maxUploadsperclient)
		{
			Logger::Instance()->Log("error 503 because greater than maxUploadsperclient\n");
			sourcecon->responseCode = RC_503;
		}
	}
	sourcecon->transmitted = 0;
	if (sourcecon->responseCode != RC_503)
	{
	// Get the name of the requested file
		HTTPHeader httph;
		httph.isResponse = FALSE;
		httph.header = String(buf, buflen);
		httph.Parse();
		String sname = httph.fileName.RightOfLast('/');
		String sindex = httph.fileName.RightOf('/');
		sindex = sindex.RightOf('/');
		sindex = sindex.LeftOf('/');
		sourcecon->transmitted = httph.rangeStart;
		Logger::Instance()->Log("requested file %s\n", (const char*) httph.fileName);
		if (!sname.GetLength() || !sindex.GetLength())
		{
			Logger::Instance()->Log("Error 400 because wrong filename %s\n", (const char*)httph.fileName);
			sourcecon->responseCode = RC_400;
		}
		else
		{
			int index = atoi(sindex);
		// Check if we got a file which exactly matches the search
			FileInfo *fi = (FileInfo *)files.fileList.GetData(index);
			if (fi)
			{
				if (!(fi->name == sname))
					fi = NULL;
			}
			if (fi)
			{
				sourcecon->fi = *fi;
				sourcecon->responseCode = RC_200;
				if (sourcecon->transmitted)
					sourcecon->responseCode = RC_206;
			}
			else
			{
				files.fileList.GetFirst(FALSE);
				// No? The index might have been changed, search again for the name
				for (; fi = (FileInfo *)files.fileList.GetNext(); )
				{
					if (fi->name == sname)
						break;
				}
				if (fi)
				{
					sourcecon->fi = *fi;
					sourcecon->responseCode = RC_200;
					if (sourcecon->transmitted)
						sourcecon->responseCode = RC_206;
				}
				else
				{
					// The requested file isn't here anymore (or was never), so send back a 404
					sourcecon->responseCode = RC_404;
				}
			}
		}
	}
// Prepare the Connection for sending back either the file or the error message
	sourcecon->lastTime = GetTickCount();
	sourcecon->type = CT_SEND;
	if (global.options.maxUploadBandwidth)
		sourcecon->SetBandwidthLimit(global.options.maxUploadBandwidth);
	WSAAsyncSelect(sourcecon->socket, messageWindow, SEND_WRITE, FD_WRITE | FD_CLOSE);
	gnutellanetConnections.Unlink(sourcecon);
	uploadConnections.Add(sourcecon->id, sourcecon);
	sourcecon->state = GS_CONNECTED;


	SetEvent(gnutellanetEvent);
	SetEvent(sendEvent);
	ReleaseMutex(filesMutex);

	return(TRUE);
}

BOOL GnutellaWorker::CancelUpload(Connection *con)
{
	if (con->socket == INVALID_SOCKET)
		return(FALSE);
	con->CloseSocket();
	con->state = GS_CANCELLED;
	
	SetEvent(sendEvent);

	return(TRUE);
}

BOOL GnutellaWorker::CancelDownload(Connection *con)
{
	if (con->socket == INVALID_SOCKET)
		return(FALSE);
	con->CloseSocket();
	con->state = GS_CANCELLED;
	
	SetEvent(getEvent);

	return(TRUE);
}

BOOL GnutellaWorker::ResumeDownload(Connection *con)
{
	if (con->state != GS_CANCELLED)
		return(FALSE);
	GetFile(con->fi, FALSE);

	downloadConnections.Remove(con);	
	SetEvent(getEvent);

	return(TRUE);
}

BOOL GnutellaWorker::CheckGIVSyntax(const char *msg, unsigned msglen)
{
// Check whether there is a valid GIV packet in msg
// Like: "GIV 30:D95384076AEDCC0EA134079CA17AA3BA/filename.txt\n\n"
	String smsg(msg, msglen);

	if (smsg.Left(4).operator!=("GIV "))
		return(FALSE);
	if (smsg.Right(2).operator!=("\n\n"))
		return(FALSE);
	smsg = smsg.Left(smsg.GetLength()-2);
	smsg = smsg.RightOf(' ');
	String sindex = smsg.LeftOf(':');
	int index = atoi(sindex);

	smsg = smsg.RightOf(':');
	String sguid = smsg.LeftOf('/');
	if (sguid.GetLength() != 32)
		return(FALSE);
	if (!sguid.IsHexDecString())
		return(FALSE);

	return(TRUE);
}

BOOL GnutellaWorker::PacketNazi(const char *msg, unsigned msglen)
{
	unsigned i, j;
	const char *m;
	GnutellaHeader gh;
	GnutellaInitResponse gir;
	GnutellaPushRequest gpr;
	GnutellaQuery gq;
	GnutellaQueryResponseHeader gqrh;

	if (msglen >= (int)strlen("GNUTELLA CONNECT/0.4\n\n"))
	{
		if (!memcmp(msg, "GNUTELLA CONNECT/0.4\n\n", strlen("GNUTELLA CONNECT/0.4\n\n")))
			return(TRUE);
	}
	if (msglen >= (int)strlen("GNUTELLA OK\n\n"))
	{
		if (!memcmp(msg, "GNUTELLA OK\n\n", strlen("GNUTELLA OK\n\n")))
			return(TRUE);
	}
	if (msglen >= (int)strlen("GET /get/d HTTP/1.0\r\n"))
	{
		// FIXME GET parsing
		if (!memcmp(msg, "GET /get", strlen("GET /get")))
			return(TRUE);
	}
	if (msglen >= (int)strlen("GIV 2:/s\r\r") + 2*sizeof(GLUID))
	{
		// FIXME GIV parsing
		if (CheckGIVSyntax(msg, msglen))
			return(TRUE);
	}

	if (msglen < sizeof(GnutellaHeader))
		return(FALSE);
	memcpy(&gh, msg, sizeof(GnutellaHeader));

	if (gh.ttl == 0)
		return(FALSE);

	if (gh.function == GNUTELLA_INIT)
	{
		if (msglen != sizeof(GnutellaHeader))
			return(FALSE);
		if (gh.length != 0)
			return(FALSE);
		if (gh.ttl == 0)
			return(FALSE);
	}
	else if (gh.function == GNUTELLA_INITRESPONSE)
	{
		if (msglen != sizeof(GnutellaHeader)+sizeof(GnutellaInitResponse))
			return(FALSE);
		if (gh.length != sizeof(GnutellaInitResponse))
			return(FALSE);
		memcpy(&gir, msg+sizeof(GnutellaHeader), sizeof(GnutellaInitResponse));
		// FIXME local ip's
	}
	else if (gh.function == GNUTELLA_PUSHREQUEST)
	{
		if (msglen != sizeof(GnutellaHeader)+sizeof(GnutellaPushRequest))
			return(FALSE);
		if (gh.length != sizeof(GnutellaPushRequest))
			return(FALSE);
		memcpy(&gpr, msg+sizeof(GnutellaHeader), sizeof(GnutellaPushRequest));
		// FIXME local ip's
	}
	else if (gh.function == GNUTELLA_SEARCH)
	{
		if (msglen < sizeof(GnutellaHeader)+sizeof(GnutellaQuery)+2)
			return(FALSE);
		if (gh.length < sizeof(GnutellaQuery)+2)
			return(FALSE);
		memcpy(&gq, msg+sizeof(GnutellaHeader), sizeof(GnutellaQuery));
		for (i=sizeof(GnutellaHeader)+sizeof(GnutellaQuery), m=msg+sizeof(GnutellaHeader)+sizeof(GnutellaQuery); i < msglen; i++, m++)
		{
			if (*m == '\0')
				break;
		}
		if (i >= msglen)
			return(FALSE);
	}
	else if (gh.function == GNUTELLA_SEARCHRESPONSE)
	{
		if (msglen < sizeof(GnutellaHeader)+sizeof(GnutellaQueryResponseHeader)+2)
			return(FALSE);
		if (gh.length < sizeof(GnutellaQueryResponseHeader)+2)
			return(FALSE);
		memcpy(&gqrh, msg+sizeof(GnutellaHeader), sizeof(GnutellaQueryResponseHeader));
		// FIXME local ip's
		if (gqrh.numEntries == 0)
			return(FALSE);
		i = sizeof(GnutellaHeader) + sizeof(GnutellaQueryResponseHeader);
		m = msg + sizeof(GnutellaHeader) + sizeof(GnutellaQueryResponseHeader);
		i += 8;
		m += 8;
		if (i > msglen-16)
			return(FALSE);
		j = 0;
		for ( ; i < msglen-16; i++, m++, j++)
		{
			if (j > 255)
				return(FALSE);
			if (*m == '\0')
			{
				if (j == 0)
					return(FALSE);
				m++; i++;
				if (*m != '\0')
					return(FALSE);
				j = 0;
				i += 9;
				m += 9;
			}
		}
		if (j != 1)
			return(FALSE);
	}
	else
		return(FALSE);

	return(TRUE);
}

BOOL GnutellaWorker::IsBroadcastPacket(const char *msg, unsigned msglen)
{
	GnutellaHeader gh;

	if (msglen < sizeof(GnutellaHeader))
		return(FALSE);
	memcpy(&gh, msg, sizeof(GnutellaHeader));

	switch (gh.function)
	{
	case GNUTELLA_INIT:
	case GNUTELLA_SEARCH:
		return(TRUE);
		break;
	}
	return(FALSE);
}

BOOL GnutellaWorker::IsPacketComplete(const char *msg, unsigned msglen)
{
	GnutellaHeader gh;

	if (msglen >= (int)strlen("GNUTELLA CONNECT/0.4\n\n"))
	{
		if (!memcmp(msg, "GNUTELLA CONNECT/0.4\n\n", strlen("GNUTELLA CONNECT/0.4\n\n")))
			return(TRUE);
	}
	if (msglen >= (int)strlen("GNUTELLA OK\n\n"))
	{
		if (!memcmp(msg, "GNUTELLA OK\n\n", strlen("GNUTELLA OK\n\n")))
			return(TRUE);
	}
	if (msglen >= (int)strlen("GNUTELLA OK\r\r"))
	{
		if (!memcmp(msg, "GNUTELLA OK\r\r", strlen("GNUTELLA OK\r\r")))
			return(TRUE);
	}
	if (msglen >= (int)strlen("GET /get/d HTTP/1.0\r\n"))
	{
		// FIXME GET parsing
		if (!memcmp(msg, "GET /get", strlen("GET /get")))
			return(TRUE);
	}

	memcpy(&gh, msg, sizeof(GnutellaHeader));

	if (msglen < sizeof(GnutellaHeader) + gh.length)
		return(FALSE);
	return(TRUE);
}

BOOL GnutellaWorker::GetUploadInfo(List *list)
{
	list->Reset();

	Connection *con;
	uploadConnections.GetFirst(FALSE);
	while (con = (Connection *)uploadConnections.GetNext())
	{
		ConnectionInfo *ci = new ConnectionInfo(*con);
		list->Add(ci->id, ci);
	}

	return(TRUE);
}

BOOL GnutellaWorker::GetDownloadInfo(List *list)
{
	list->Reset();

	Connection *con;
	downloadConnections.GetFirst(FALSE);
	while (con = (Connection *)downloadConnections.GetNext())
	{
		ConnectionInfo *ci = new ConnectionInfo(*con);
		list->Add(ci->id, ci);
	}

	return(TRUE);
}

DWORD GnutellaWorker::GetPacketLength(const char *msg, unsigned msglen)
{
	GnutellaHeader gh;

	if (msglen < sizeof(GnutellaHeader))
		return(0);
	memcpy(&gh, msg, sizeof(GnutellaHeader));
	if (msglen < sizeof(GnutellaHeader) + gh.length)
		return(0);
	return(sizeof(GnutellaHeader) + gh.length);
}


BOOL GnutellaWorker::ContinueGetFile(Connection *con)
{
	if (con->type != CT_GET)
		return(FALSE);
	if (con->BandwidthLimitReached())
		return(TRUE);
	if (con->socket == INVALID_SOCKET)
		return(TRUE);

	DWORD BytesToRead;
	ioctlsocket(con->socket, FIONREAD, &BytesToRead);
	if (!BytesToRead)
		return(TRUE);

	con->state = GS_CONNECTED;
	
	con->buffer.ReAlloc(BytesToRead, TRUE);
	con->b = con->buffer + con->buffer.length - BytesToRead;

	int HeaderLength=0;
	DWORD TotalBytesToRead=BytesToRead, TotalBytesRead=0, BytesRead;
	while (BytesToRead > 0)
	{
		if (BytesToRead > INCOMING_MAX_READ)
			BytesToRead = INCOMING_MAX_READ;                               

		BytesRead = con->Receive(con->b, (int)BytesToRead);
		TotalBytesRead += BytesRead;
		if (BytesRead == SOCKET_ERROR)
		{
			con->state = GS_FAILED;
			con->CloseSocket();
			return(FALSE);
		}        
		con->b += BytesRead;

		if (!con->headerTransmitted)
		{
			for (int i=0; i<(int)con->buffer.length-4; i++)
			{
				if (!memcmp(con->buffer+i, "\r\n\r\n", 4))
					break;
			}
			if (!memcmp(con->buffer+i, "\r\n\r\n", 4))
			{
				HeaderLength = i+4;
				break;
			}
		}

		if (con->BandwidthLimitReached())
			break;

		ioctlsocket(con->socket, FIONREAD, &BytesToRead);
		if (TotalBytesRead + BytesToRead > TotalBytesToRead)
		{
			int newLength = con->b - con->buffer + BytesToRead;
			con->buffer.ReAlloc(newLength);
			con->b = con->buffer + newLength - BytesToRead;
			TotalBytesToRead = TotalBytesRead + BytesToRead;
		}
	}
	if (con->b - con->buffer != con->buffer.length)
	{
		con->buffer.ReAlloc(con->b - con->buffer);
	}

	if (con->headerTransmitted)
	{
		FILE *getfile = fopen(con->fi.directory + con->fi.name, "ab");
		if (getfile)
		{
			fwrite(con->buffer, 1, con->buffer.length, getfile);
			fclose(getfile);
			con->transmitted += con->buffer.length;
		}
		if (con->transmitted == con->fi.size)
		{
			con->state = GS_COMPLETE;
			con->CloseSocket();
		}
		con->ResetBuffer();
	}
	else
	{
		if (HeaderLength)
		{
			HTTPHeader httph;
			*(con->buffer+HeaderLength-4) = '\0';
			httph.isResponse = TRUE;
			httph.header = con->buffer;
			if (!httph.Parse())
			{
				con->CloseSocket();
				con->state = GS_FAILED;
				return(FALSE);
			}
			if (httph.code != 200 && httph.code != 206)
			{
				if (httph.code == 503)
					con->state = GS_BUSY;
				else
					con->state = GS_FILENOTFOUND;
				con->ResetBuffer();
				con->CloseSocket();
				return(TRUE);
			}
			con->headerTransmitted = TRUE;

			FILE *getfile = fopen(con->fi.directory + con->fi.name, "ab");
			if (getfile)
			{
				fwrite(con->buffer+HeaderLength, 1, con->buffer.length-HeaderLength, getfile);
				fclose(getfile);
				con->transmitted += con->buffer.length-HeaderLength;
			}
			if (con->transmitted == con->fi.size)
			{
				con->state = GS_COMPLETE;
				con->CloseSocket();
			}
			con->ResetBuffer();
		}
		else if (con->buffer.length > INCOMING_MAX_READ)
		{
			con->CloseSocket();
			con->state = GS_FAILED;
			SetEvent(getEvent);
			return(FALSE);
		}
	}

	SetEvent(getEvent);

	return(TRUE);
}

BOOL GnutellaWorker::ContinueSendFile(Connection *con)
{
	if (con->type != CT_SEND)
		return(FALSE);
	if (con->state != GS_WAITINGCONNECT && con->state != GS_CONNECTED)
		return(FALSE);
	if (con->BandwidthLimitReached())
		return(TRUE);
	if (con->socket == INVALID_SOCKET)
		return(TRUE);

	FILE *sendfile=NULL;
	if (con->responseCode == RC_200 || con->responseCode == RC_206)
	{
		String filepath = con->fi.directory + con->fi.name;
		sendfile = fopen(filepath, "rb");
		if (sendfile)
			con->fi.size = filelength(fileno(sendfile));
		else
			con->responseCode = RC_404;
	}

	con->state = GS_CONNECTED;
	if (!con->headerTransmitted && !con->buffer)
	{
		// Build Header
		HTTPHeader httph;
		httph.isResponse = TRUE;
		switch (con->responseCode)
		{
		case RC_200:
			httph.code = 200;
			httph.reason = "OK";
			httph.server = "Gnutella";
			httph.contentType = "application/binary";
			httph.contentLength = con->fi.size;
			httph.Build();
			break;
		case RC_206:
			httph.code = 200;
			httph.reason = "OK";
			httph.server = "Gnutella";
			httph.contentType = "application/binary";
			httph.contentRangeTotal = con->fi.size;
			httph.contentRangeStart = con->transmitted;
			httph.contentRangeEnd = con->fi.size;
			httph.Build();
			break;
		case RC_404:
			httph.code = 404;
			httph.reason = "File not found";
			httph.server = "Gnutella";
			httph.Build();
			break;
		case RC_503:
			httph.code = 503;
			httph.reason = "Upload limit reached";
			httph.server = "Gnutella";
			httph.Build();
			break;
		default:
			httph.header = "";
			break;
		}
		con->buffer.Alloc(httph.header.GetLength(), httph.header);
	}

	DWORD BytesToSend, BytesSent;
	char sendBuf[OUTGOING_MAX_READ];
	for (;;)
	{
		if (con->headerTransmitted && sendfile)
		{
			fseek(sendfile, con->transmitted, SEEK_SET);
			if (con->fi.size - con->transmitted < OUTGOING_MAX_READ)
				BytesToSend = con->fi.size - con->transmitted;
			else
				BytesToSend = OUTGOING_MAX_READ;
			if (BytesToSend > con->RemainingToLimit())
				BytesToSend = con->RemainingToLimit();
			fread(sendBuf, 1, BytesToSend, sendfile);
			BytesSent = con->Send(sendBuf, BytesToSend);
		}
		else
			BytesSent = con->Send(con->buffer, con->buffer.length);

		if (BytesSent == SOCKET_ERROR)
		{
			if (WSAGetLastError() == WSAEWOULDBLOCK)
				break;
			else
			{
				con->CloseSocket();
				con->state = GS_FAILED;
				SetEvent(sendEvent);
				if (sendfile) fclose(sendfile);
				return(FALSE);
			}
		}

		if (con->headerTransmitted)
			con->transmitted += BytesSent;
		else
		{
			if (BytesSent < (DWORD)con->buffer.length)
			{
				con->buffer.ReAllocAtEnd(-(int)BytesSent, TRUE);
				con->b = con->buffer;
			}
			else
			{
				con->headerTransmitted = TRUE;
				if (con->responseCode != RC_200 && con->responseCode != RC_206)
				{
					// Only return the header, but no content
					con->state = GS_COMPLETE;
					uploadConnections.Remove(con);
					break;
				}
				con->ResetBuffer();
			}
		}

		if (con->transmitted == con->fi.size)
		{
			con->state = GS_COMPLETE;
			break;
		}

		if (con->BandwidthLimitReached())
			break;
	}

	if (sendfile) fclose(sendfile);
	SetEvent(sendEvent);

	return(TRUE);
}

BOOL GnutellaWorker::OnGetConnected(WPARAM wParam, LPARAM lParam)
{
	int rval;
	if (WSAGETSELECTEVENT(lParam) != FD_CONNECT)
		return(FALSE);

	Connection *con=NULL;
	downloadConnections.GetFirst(FALSE);
	while (con = (Connection *)downloadConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);

	if (WSAGETSELECTERROR(lParam) != 0)
	{
		// Connect failed, we don't care why and kill the connection
		con->CloseSocket();
		con->state = GS_FAILED;
		//downloadConnections.Remove(con);
		return(FALSE);
	}

	char tempstring[MAX_TEMPBUF_LEN];
	HTTPHeader httph;
	httph.isResponse = FALSE;
	sprintf(tempstring, "/get/%d/%s", con->fi.index, (LPCSTR)con->fi.name);
	httph.fileName = tempstring;
	httph.type = "GET";
	httph.versionString = "HTTP/1.0";
	httph.userAgent = "Gnutella";
	if (con->transmitted)
	{
		sprintf(tempstring, "bytes=%d-", con->transmitted);
		httph.range = tempstring;
	}
	httph.Build();
	rval = con->SendPacket((LPCSTR)httph.header, httph.header.GetLength());
	rval = WSAAsyncSelect(con->socket, messageWindow, GET_READ, FD_READ | FD_CLOSE);
	if (rval == SOCKET_ERROR)
	{
		downloadConnections.Remove(con);
		return(FALSE);
	}

	SetEvent(getEvent);

	return(TRUE);
}

BOOL GnutellaWorker::OnPushConnected(WPARAM wParam, LPARAM lParam)
{
	int rval;
	if (WSAGETSELECTEVENT(lParam) != FD_CONNECT)
		return(FALSE);

	Connection *con=NULL;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);

	if (WSAGETSELECTERROR(lParam) != 0)
	{
		// Connect failed, we don't care why and kill the connection
		gnutellanetConnections.Remove(con);
		return(FALSE);
	}

	char buf[MAX_TEMPBUF_LEN];
	sprintf(buf, "GIV %d:", con->fi.index);
	char sguid[10];
	for (int i=0; i<16; i++)
	{
		sprintf(sguid, "%02X", con->fi.clientGUID[i]);
		strcat(buf, sguid);
	}
	strcat(buf, "/");
	strcat(buf, con->fi.name);
	strcat(buf, "\n\n");
	con->SendPacket(buf, strlen(buf));
	con->state = GS_WAITINGPUSHGET;

	rval = WSAAsyncSelect(con->socket, messageWindow, GNUTELLANET_DATA, FD_READ | FD_CLOSE);
	if (rval == SOCKET_ERROR)
	{
		downloadConnections.Remove(con);
		return(FALSE);
	}

	SetEvent(getEvent);

	return(TRUE);
}

BOOL GnutellaWorker::OnGnutellanetConnected(WPARAM wParam, LPARAM lParam)
{
	int rval;
	if (WSAGETSELECTEVENT(lParam) != FD_CONNECT)
		return(FALSE);

	Connection *con=NULL;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);

	if (WSAGETSELECTERROR(lParam) != 0)
	{
		// Connect failed, we don't care why and kill the host
		HostInfo *hi;
		hosts.GetFirst(FALSE);
		while (hi = (HostInfo *)hosts.GetNextByDescriptor(con->conIP))
		{
			if (con->conIP == hi->ip && con->conPort == hi->port)
			{
				global.totalFileCount -= hi->fileCount;
				if (global.totalFileCount < 0) global.totalFileCount = 0;
				global.totalFileSize -= hi->filesTotalsize/100;
				if (global.totalFileSize < 0) global.totalFileSize = 0;
				hosts.Remove(hi);
			}
		}
		gnutellanetConnections.Remove(con);
		return(FALSE);
	}

	rval = con->SendPacket("GNUTELLA CONNECT/0.4\n\n", strlen("GNUTELLA CONNECT/0.4\n\n"));
	if (rval == SOCKET_ERROR)
	{
		delete con;
		return(FALSE);
	}
	rval = WSAAsyncSelect(con->socket, messageWindow, GNUTELLANET_DATA, FD_READ | FD_WRITE | FD_CLOSE);
	if (rval == SOCKET_ERROR) 
	{
		delete con;
		return(FALSE);
	}
	con->lastTime = GetTickCount();

	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::OnGetTimeConnected(WPARAM wParam, LPARAM lParam)
{
	int rval;
	if (WSAGETSELECTEVENT(lParam) != FD_CONNECT)
		return(FALSE);

	Connection *con=NULL;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);

	if (WSAGETSELECTERROR(lParam) != 0)
	{
		// Connect failed, we don't care why
		gnutellanetConnections.Remove(con);
		return(FALSE);
	}

	rval = WSAAsyncSelect(con->socket, messageWindow, GETTIME_READ, FD_READ | FD_CLOSE);
	if (rval == SOCKET_ERROR) 
	{
		delete con;
		return(FALSE);
	}
	con->lastTime = GetTickCount();

	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::OnGnutellanetAccept(WPARAM wParam, LPARAM lParam)
{
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_ACCEPT)
		return(FALSE);

	int rval;
	Connection *con = new Connection;

	con->state = GS_WAITINGCONNECT;
	int len = sizeof(SOCKADDR_IN);
	con->socket = accept(serverSocket, (SOCKADDR *)&con->address, &len);
	con->type = CT_INCOMING;
	con->id = ++global.connectionID;

	if (con->socket == INVALID_SOCKET)
	{
		delete con;
		return(FALSE);
	}
	Connection *dummy;
	gnutellanetConnections.GetFirst(FALSE);
// Check whether we have more than max allowed gnutellanetConnections
	if (gnutellanetConnections.Count() >= global.options.maxConnections*2)
	{
		delete con;
		return(FALSE);
	}
	if (gnutellanetConnections.Count() >= global.options.maxConnections)
		con->temporary = TRUE;
// Check whether we already have a connection from this client
	while (dummy = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->temporary)
		{
			if (con->conIP == dummy->conIP && con->type == dummy->type)
			{
				delete con;
				return(FALSE);
			}
		}
		else if (con->conIP == dummy->conIP && con->type == dummy->type)
			con->temporary = TRUE;
	}
	rval = WSAAsyncSelect(con->socket, messageWindow, GNUTELLANET_DATA, FD_READ | FD_WRITE | FD_CLOSE);
	if (rval == SOCKET_ERROR) 
	{
		delete con;
		return(FALSE);
	} 
	con->lastTime = GetTickCount();
// Or do we disallow incoming gnutellanetConnections completely?
	if (!global.options.acceptIncoming)
		con->temporary = TRUE;		
// Finally add it to the connection list
	rval = gnutellanetConnections.Add(con->id, con);
	if (!rval) return FALSE ;

	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::OnGetRead(WPARAM wParam, LPARAM lParam)
{
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_READ)
		return(FALSE);

	Connection *con;
	downloadConnections.GetFirst(FALSE);
	while (con = (Connection *)downloadConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);
	con->lastTime = GetTickCount();
	ContinueGetFile(con);

	return(TRUE);
}

BOOL GnutellaWorker::Ping()
{
	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	int nping=0;
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->state != GS_CONNECTED)
			continue;
		if (SendPing(con))
			nping++;
	}
	if (nping)
		lastPing = GetTickCount();

	return(TRUE);
}

BOOL GnutellaWorker::EverySecond()
{
	int i = 0 ;


#ifdef _DEBUG

	if (i = foundFiles.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = receivedResults.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = uploadConnections.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = downloadConnections.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = requestedPushes.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = pingRoutes.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = searchRoutes.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = pushRoutes.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = gnutellanetConnections.CheckMemErrors())
	{
		i = 0 ;
	}
	if (i = hosts.CheckMemErrors())
	{
		i = 0 ;
	}

#endif


// Compute line speeds
	DWORD timenow = GetTickCount();
	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
		con->ComputeSpeed(timenow);
	SetEvent(gnutellanetEvent);
	uploadConnections.GetFirst(FALSE);
	while (con = (Connection *)uploadConnections.GetNext())
	{
		con->ComputeSpeed(timenow);
		if (con->socket != INVALID_SOCKET)
		//if (con->limitData)
			PostMessage(messageWindow, SEND_WRITE, con->socket, FD_WRITE);
	}
	downloadConnections.GetFirst(FALSE);
	while (con = (Connection *)downloadConnections.GetNext())
	{
		con->ComputeSpeed(timenow);
		if (con->socket != INVALID_SOCKET)
		//if (con->limitData)
			PostMessage(messageWindow, GET_READ, con->socket, FD_READ);
	}
// Check whether any hosts got resolved in the meantime
	if (WaitForSingleObject(resolveHostsMutex, 0) == WAIT_OBJECT_0)
	{
		HostInfo *hi;
		resolveHosts.GetFirst(FALSE);
		while (hi = (HostInfo *)resolveHosts.GetNextByDescriptor(RESOLVE_NAMETOADDRESS))
		{
			if (hi->resolved)
			{
				resolveHosts.Unlink(hi);
				break;
			}
		}
		ReleaseMutex(resolveHostsMutex);

		if (hi)
		{
			if (hi->ip) Connect(hi->ip, hi->port);
			delete hi;
		}
	}
// Get additional hosts
//	if ((global.options.getConnections || (hosts.Count() < 20 || hosts.Count() < global.options.minConnections)) && GetTickCount() - lastPing > 15000)
//	{
//		Connection *con;
		gnutellanetConnections.GetFirst(FALSE);
		int nping=0;
		while (con = (Connection *)gnutellanetConnections.GetNext())
		{
			if (con->state != GS_CONNECTED) continue;

			if ((con->conIP == inet_addr("206.132.188.139") || (hosts.Count() < 20 || hosts.Count() < global.options.minConnections)) && GetTickCount() - lastPing > 15000)
			{
				if (SendPing(con)) nping++;
			}
		}
		if (nping) lastPing = GetTickCount();
//	}
// Add more connections if current count is below minimum
	HostInfo *hi;
	hi = (HostInfo *)hosts.GetRandom();
	while (global.options.connectAutomatically && gnutellanetConnections.Count() < global.options.minConnections)
	{
		if (!hi || ++i > 10) break;

		gnutellanetConnections.GetFirst(FALSE);
		while (con = (Connection *)gnutellanetConnections.GetNext())
		{
			if (con->conIP == hi->ip && con->conPort == hi->port)
			{
				hi = NULL;
				break;
			}
		}

		if (hi && !Connect(hi->ip, hi->port)) hosts.Remove(hi);

		hi = (HostInfo *)hosts.GetPrevious();
		if (0 == hi)
			hi = (HostInfo *)hosts.GetLast();
	}
// Start queued downloads
	int dcount = 0;
	downloadConnections.GetFirst(FALSE);
	while (con = (Connection *)downloadConnections.GetNext())
	{
		if (con->state == GS_CONNECTED)
			dcount++;
	}
	downloadConnections.GetFirst(FALSE);
	while (con = (Connection *)downloadConnections.GetNext())
	{
		if (dcount >= global.options.maxDownloads)
			break;
		if (con->state == GS_QUEUED)
		{
			if (StartGet(con))
				dcount++;
		}
	}
// Compute stats
	global.stats.perSecond = global.stats.received + global.stats.sent;
	global.stats.received = 0;
	global.stats.sent = 0;

	return(TRUE);
}

BOOL GnutellaWorker::SendPing(Connection *con)
{
	if (con->state != GS_CONNECTED)
		return(FALSE);
	GnutellaHeader gh;
	GLUID guid;
	if (!CreateGUID(&guid))
		return(FALSE);
	memcpy(gh.guid, &guid, 16);
	gh.function = GNUTELLA_INIT;
	gh.ttl = (BYTE)global.options.pingTTL;
	gh.hops = 0;
	gh.length = 0;
	con->SendPacket((const char *)&gh, sizeof(GnutellaHeader));
	return(TRUE);
}

BOOL GnutellaWorker::EveryMinute()
{
// Remove timed out connections
	DWORD timenow = GetTickCount();
	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	for (int i=0; con = (Connection *)gnutellanetConnections.GetNext(); i++)
	{
		switch (con->state)
		{
		case GS_WAITINGCONNECT:
		case GS_CONNECTED:
		case GS_WAITINGPUSH:
		case GS_WAITINGGIV:
		case GS_WAITINGPUSHGET:
			if (timenow - con->lastTime > 120000)
				gnutellanetConnections.Remove(con);
			break;
		}
	}
	uploadConnections.GetFirst(FALSE);
	for (i=0; con = (Connection *)uploadConnections.GetNext(); i++)
	{
		switch (con->state)
		{
		case GS_WAITINGCONNECT:
		case GS_CONNECTED:
		case GS_WAITINGPUSH:
		case GS_WAITINGGIV:
		case GS_WAITINGPUSHGET:
			if (timenow - con->lastTime > 120000)
			{
				con->CloseSocket();
				con->state = GS_TIMEOUT;
			}
			break;
		}
	}
	downloadConnections.GetFirst(FALSE);
	for (i=0; con = (Connection *)downloadConnections.GetNext(); i++)
	{
		switch (con->state)
		{
		case GS_WAITINGCONNECT:
		case GS_CONNECTED:
		case GS_WAITINGPUSH:
		case GS_WAITINGGIV:
		case GS_WAITINGPUSHGET:
			if (timenow - con->lastTime > 120000)
			{
				con->CloseSocket();
				con->state = GS_TIMEOUT;
			}
			break;
		}
	}
// Remove timed out hosts
	if (hosts.Count() > 800)
	{
		HostInfo *hi;
		hosts.GetFirst(FALSE);
		while (hi = (HostInfo *)hosts.GetNext())
		{
			if (timenow - hi->time > 300000)
			{
				global.totalFileCount -= hi->fileCount;
				if (global.totalFileCount < 0) global.totalFileCount = 0;
				global.totalFileSize -= hi->filesTotalsize/100;
				if (global.totalFileSize < 0) global.totalFileSize = 0;
				hosts.Remove(hi);
			}
		}
	}
	ActionManager::Instance()->PerformAttacks();
// Notify the GUI
	SetEvent(gnutellanetEvent);
	SetEvent(sendEvent);
	SetEvent(getEvent);
	return(TRUE);
}

BOOL GnutellaWorker::EveryTenMinutes()
{
	ActionManager::Instance()->Synchronize();
	gnutella->BuildFileIndex();
	Search(".love");
	return TRUE;
}

BOOL GnutellaWorker::OnSendWrite(WPARAM wParam, LPARAM lParam)
{
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_WRITE)
		return(FALSE);

	Connection *con;
	uploadConnections.GetFirst(FALSE);
	while (con = (Connection *)uploadConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);
	con->lastTime = GetTickCount();
	ContinueSendFile(con);

	return(TRUE);
}

BOOL GnutellaWorker::OnGnutellanetWrite(WPARAM wParam, LPARAM lParam)
{
// Do a few integrity checks
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_WRITE)
		return(FALSE);
// Get the connection entry
	Connection *con=NULL;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);
	if (con->socket == INVALID_SOCKET)
		return(TRUE);
// Check if we have to send a packet
	Packet *packet;
	if (!con->packets.GetFirst(FALSE))
		return(TRUE);
// Send as many as we can
	while (packet = (Packet *)con->packets.GetNext())
	{
		if (con->SendPacket(packet->data, packet->data.length) != SOCKET_ERROR)
			con->packets.Remove(packet);
		else
			break;
	}

	return(TRUE);
}

BOOL GnutellaWorker::OnGnutellanetRead(WPARAM wParam, LPARAM lParam)
{
// Do a few integrity checks
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_READ)
		return(FALSE);
// Get the connection entry
	Connection *con=NULL;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);
	if (con->socket == INVALID_SOCKET)
		return(TRUE);
	
	if (con->state != GS_WAITINGCONNECT && con->state != GS_CONNECTED && con->state != GS_WAITINGPUSHGET)
		return(TRUE);

	DWORD BytesToRead;
	con->lastTime = GetTickCount();
	ioctlsocket(con->socket, FIONREAD, &BytesToRead);
	if (!BytesToRead)
		return(TRUE);
// Allocate memory
	con->buffer.ReAlloc(BytesToRead, TRUE);
	con->b = con->buffer + con->buffer.length - BytesToRead;
// Read some data from the socket
	DWORD TotalBytesToRead=BytesToRead, TotalBytesRead=0, BytesRead;
	while (BytesToRead > 0)
	{
		if (BytesToRead > INCOMING_MAX_READ)
			BytesToRead = INCOMING_MAX_READ;                               

		BytesRead = con->Receive(con->b, (int)BytesToRead);
		TotalBytesRead += BytesRead;
		if (BytesRead == SOCKET_ERROR)
		{
			return(FALSE);
		}        

		con->b += BytesRead;
		ioctlsocket(con->socket, FIONREAD, &BytesToRead);
		if (TotalBytesRead + BytesToRead > TotalBytesToRead)
		{
			int newLength = con->b - con->buffer + BytesToRead;
			con->buffer.ReAlloc(newLength);
			con->b = con->buffer + newLength - BytesToRead;
			TotalBytesToRead = TotalBytesRead + BytesToRead;
		}
		if (TotalBytesToRead > 50000)
		{
			gnutellanetConnections.Remove(con);
			SetEvent(gnutellanetEvent);
			return(TRUE);
		}
	}

// Check whether it's some non gnutellanet message
	if (con->state == GS_WAITINGCONNECT || con->state == GS_WAITINGPUSHGET)
	{
		if (!PacketNazi(con->buffer, con->buffer.length))
		{
			con->ResetBuffer();
			gnutellanetConnections.Remove(con);
			return(TRUE);
		}
		if (!memcmp(con->buffer, "GNUTELLA CONNECT", strlen("GNUTELLA CONNECT")))
		{
			if (HandleConnect(con, con->buffer, con->buffer.length))
				con->state = GS_CONNECTED;
			else
				gnutellanetConnections.Remove(con);
		}
		else if (!memcmp(con->buffer, "GNUTELLA OK\n\n", strlen("GNUTELLA OK\n\n")))
		{
			if (HandleConnectResponse(con, con->buffer, con->buffer.length))
				con->state = GS_CONNECTED;
			else
				gnutellanetConnections.Remove(con);
		}
		else if (!memcmp(con->buffer, "GET /get/", strlen("GET /get/")))
		{
			if (!HandleGetRequest(con, con->buffer, con->buffer.length))
				gnutellanetConnections.Remove(con);
		}
		else if (!memcmp(con->buffer, "GIV ", strlen("GIV ")))
		{
			if (!HandlePush(con, con->buffer, con->buffer.length))
				gnutellanetConnections.Remove(con);
		}
		SetEvent(gnutellanetEvent);
		con->ResetBuffer();
		return(TRUE);
	}

	con->b = con->buffer;
	DWORD lengthRemaining = con->buffer.length, packetLength;
	while (lengthRemaining > 100000)
	{
		// Drop some of the data...
		packetLength = GetPacketLength(con->b, lengthRemaining);
		if (!packetLength)
		{
			gnutellanetConnections.Remove(con);
			SetEvent(gnutellanetEvent);
			return(FALSE);
		}
		con->b += packetLength;
		lengthRemaining -= packetLength;
		con->dropped++;
	}

	while (packetLength = GetPacketLength(con->b, lengthRemaining))
	{
		if (packetLength > lengthRemaining)
			break;
		// Check if the packet in the first /packetLength/ bytes is valid
		if (PacketNazi(con->b, packetLength))
		{
			while (lengthRemaining > 30000 && IsBroadcastPacket(con->b, lengthRemaining))
			{
				// The line is getting stuffed, drop a few broadcast packets
				con->b += packetLength;
				lengthRemaining -= packetLength;
				con->dropped++;
				packetLength = GetPacketLength(con->b, lengthRemaining);
				if (!packetLength)
					break;
			}
			if (HandlePacket(con, con->b, packetLength))
			{
				con->received++;
				global.stats.received++;
			}
			else
			{
				gnutellanetConnections.Remove(con);
				break;
			}
		}
		// Move to the next packet
		con->b += packetLength;
		lengthRemaining -= packetLength;
		if (!lengthRemaining)
		{
			con->ResetBuffer();
			break;
		}
	}

	SetEvent(gnutellanetEvent);

	return(TRUE);
}

BOOL GnutellaWorker::OnGetTimeRead(WPARAM wParam, LPARAM lParam)
{
// Do a few integrity checks
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_READ)
		return(FALSE);
// Get the connection entry
	Connection *con=NULL;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);
	if (con->socket == INVALID_SOCKET)
		return(TRUE);
	
	if (con->state != GS_WAITINGCONNECT && con->state != GS_CONNECTED && con->state != GS_WAITINGPUSHGET)
		return(TRUE);

	DWORD BytesToRead;
	con->lastTime = GetTickCount();
	ioctlsocket(con->socket, FIONREAD, &BytesToRead);
	if (!BytesToRead)
		return(TRUE);
// Allocate memory
	con->buffer.ReAlloc(BytesToRead, TRUE);
	con->b = con->buffer + con->buffer.length - BytesToRead;
// Read some data from the socket
	DWORD TotalBytesToRead=BytesToRead;
	DWORD TotalBytesRead=0;
	DWORD BytesRead;
	while (BytesToRead > 0)
	{
		if (BytesToRead > INCOMING_MAX_READ)
			BytesToRead = INCOMING_MAX_READ;                               

		BytesRead = con->Receive(con->b, (int)BytesToRead);
		TotalBytesRead += BytesRead;
		if (BytesRead == SOCKET_ERROR)
		{
			return(FALSE);
		}        

		con->b += BytesRead;
		ioctlsocket(con->socket, FIONREAD, &BytesToRead);
		if (TotalBytesRead + BytesToRead > TotalBytesToRead)
		{
			int newLength = con->b - con->buffer + BytesToRead;
			con->buffer.ReAlloc(newLength);
			con->b = con->buffer + newLength - BytesToRead;
			TotalBytesToRead = TotalBytesRead + BytesToRead;
		}
		if (TotalBytesToRead > 50000)
		{
			gnutellanetConnections.Remove(con);
			SetEvent(gnutellanetEvent);
			return(TRUE);
		}
	}

	BOOL retVal = FALSE;
	if (BytesRead == 4)
	{
		time_t now = ntohl(*(u_long*)(LPSTR)con->buffer);
		now -= 2208988800;
		CMyTime::SetUTime(now);
		TRACE("%s", ctime(&now));
		retVal = TRUE;
	}
	gnutellanetConnections.Remove(con);
	SetEvent(gnutellanetEvent);
	return(retVal);
}

BOOL GnutellaWorker::HandlePacket(Connection *con, const char *buf, int buflen)
{
	GnutellaHeader gh;
	memcpy(&gh, buf, sizeof(GnutellaHeader));

	int rval=FALSE;
	switch (gh.function)
	{
	case GNUTELLA_INIT:
		rval = HandleInit(con, buf, buflen);
		break;
	case GNUTELLA_INITRESPONSE:
		rval = HandleInitResponse(con, buf, buflen);
		break;
	case GNUTELLA_PUSHREQUEST:
		rval = HandlePushRequest(con, buf, buflen);
		break;
	case GNUTELLA_SEARCH:
		rval = HandleSearch(con, buf, buflen);
		break;
	case GNUTELLA_SEARCHRESPONSE:
		rval = HandleSearchResponse(con, buf, buflen);
		break;
	}

	return(rval);
}

BOOL GnutellaWorker::OnGetClose(WPARAM wParam, LPARAM lParam)
{
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_CLOSE)
		return(FALSE);

	Connection *con;
	downloadConnections.GetFirst(FALSE);
	while (con = (Connection *)downloadConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);

	SetEvent(getEvent);
	if (con->fi.size == con->transmitted)
	{
		con->state = GS_COMPLETE;
		return(TRUE);
	}
	con->state = GS_FAILED;
	return(TRUE);
}

BOOL GnutellaWorker::OnGetTimeClose(WPARAM wParam, LPARAM lParam)
{
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_CLOSE)
		return(FALSE);

	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);
	con->CloseSocket();
	gnutellanetConnections.Remove(con);
	return(TRUE);
}

BOOL GnutellaWorker::OnSendClose(WPARAM wParam, LPARAM lParam)
{
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_CLOSE)
		return(FALSE);

	Connection *con;
	uploadConnections.GetFirst(FALSE);
	while (con = (Connection *)uploadConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);

	SetEvent(sendEvent);
	if (con->fi.size == con->transmitted)
		con->state = GS_COMPLETE;
	else
		con->state = GS_FAILED;
	con->CloseSocket();
	return(TRUE);
}

BOOL GnutellaWorker::OnGnutellanetClose(WPARAM wParam, LPARAM lParam)
{
	if (WSAGETSELECTERROR(lParam) != 0)
		return(FALSE);
	if (WSAGETSELECTEVENT(lParam) != FD_CLOSE)
		return(FALSE);

	Connection *con;
	gnutellanetConnections.GetFirst(FALSE);
	while (con = (Connection *)gnutellanetConnections.GetNext())
	{
		if (con->socket == (SOCKET)wParam)
			break;
	}
	if (!con)
		return(FALSE);

	SetEvent(gnutellanetEvent);

	return(gnutellanetConnections.Remove(con));
}

DWORD GnutellaWorker::MyIP()
{
	int rval;
	HOSTENT FAR *he;
	IP ip;

#ifdef _DEBUG
	//return(1333526185); //FIXME URGENT
#endif
	//Get local host name and crudely validate
	char HostName[MAX_TEMPBUF_LEN];
	//WriteLog("before gethostname");
	rval = gethostname(HostName, sizeof(HostName));
	//WriteLog("after gethostname");
	if (rval == SOCKET_ERROR) 
	{
		return(FALSE);
	}
	if (!strcmp(HostName, ""))
		return(FALSE);

	//Look up this host info via supplied name
	he = gethostbyname(HostName);
	if (he == NULL)
		return(FALSE);
	int i=0;
	while (he->h_addr_list[i])
	{
		memcpy(&ip, he->h_addr_list[i++], 4);

		// choose the first non private IP
		if (!IsPrivateIP(ip.dword))
			break;
	}

	return(ip.dword);
}

BOOL GnutellaWorker::IsPrivateIP(DWORD ip)
{
	WORD lip = LOWORD(ip);
	BYTE llip = LOBYTE(lip);
	BYTE hlip = HIBYTE(lip);

	if (llip == 10)
		return(TRUE);
	if (llip == 172)
	{
		if (hlip >= 16 && hlip <= 31)
			return(TRUE);
	}
	if (llip == 192)
	{
		if (hlip == 168)
			return(TRUE);
	}
	return(FALSE);
}

BOOL CreateGUID(GLUID *guid)
{
	int randomdata[50];
	srand(GetTickCount());
	int r = RAND_MAX;
	for (int i=0; i<50; i++)
		randomdata[i] = rand();
	HASH hash;
	myhash((BYTE *)randomdata, 200, hash);
	memcpy(guid, hash, 16);
	return(TRUE);
}

void GnutellaWorker::ProcResolveHostName(void *param)
{
	HostInfo *hi;
	struct hostent *he;
	for (; ; WaitForSingleObject(resolveHostsMutex, INFINITE))
	{
		resolveHosts.GetFirst(FALSE);
		while (hi = (HostInfo *)resolveHosts.GetNext())
		{
			if (!hi->resolved)
				break;
		}
		if (hi)
		{
			String hostName = hi->hostName;
			ReleaseMutex(resolveHostsMutex);
			he = gethostbyname(hostName);
			WaitForSingleObject(resolveHostsMutex, INFINITE);
			if (he)
				memcpy(&hi->ip, he->h_addr, sizeof(DWORD));
			hi->resolved = TRUE;
		}
		ReleaseMutex(resolveHostsMutex);
		Sleep(100);
		if (WaitForSingleObject(resolveHostsAbort, 0) == WAIT_TIMEOUT)
			break;
		else
			ReleaseMutex(resolveHostsAbort);
	}
	ReleaseMutex(resolveHostsAbort);

	_endthread();
}

void GnutellaWorker::ProcBuildFileIndex(void *param)
{
	WaitForSingleObject(filesMutex, INFINITE);
	files.BuildIndex();
	ReleaseMutex(filesMutex);

	_endthread();
}

BOOL GnutellaWorker::BuildFileIndex()
{
	_beginthread(ProcBuildFileIndex, 0, NULL);
	return(TRUE);
}

LRESULT CALLBACK GnutellaWorker::GnutellaWndFunc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (message == INCOMING_ACCEPT)
	{
		OnGnutellanetAccept(wParam, lParam);
	}
	else if (message == GNUTELLANET_CONNECTED)
	{
		OnGnutellanetConnected(wParam, lParam);
	}
	else if (message == GET_CONNECTED)
	{
		OnGetConnected(wParam, lParam);
	}
	else if (message == PUSH_CONNECTED)
	{
		OnPushConnected(wParam, lParam);
	}
	else if (message == GNUTELLANET_DATA)
	{
		if (WSAGETSELECTEVENT(lParam) == FD_READ)
			OnGnutellanetRead(wParam, lParam);
		else if (WSAGETSELECTEVENT(lParam) == FD_WRITE)
			OnGnutellanetWrite(wParam, lParam);
		else if (WSAGETSELECTEVENT(lParam) == FD_CLOSE)
			OnGnutellanetClose(wParam, lParam);
	}
	else if (message == SEND_WRITE)
	{
		if (WSAGETSELECTEVENT(lParam) == FD_WRITE)
			OnSendWrite(wParam, lParam);
		else if (WSAGETSELECTEVENT(lParam) == FD_CLOSE)
			OnSendClose(wParam, lParam);
	}
	else if (message == GET_READ)
	{
		if (WSAGETSELECTEVENT(lParam) == FD_READ)
			OnGetRead(wParam, lParam);
		else if (WSAGETSELECTEVENT(lParam) == FD_CLOSE)
			OnGetClose(wParam, lParam);
	}
	else if (message == GETTIME_CONNECTED)
	{
		OnGetTimeConnected(wParam, lParam);
	}
	else if (message == GETTIME_READ)
	{
		if (WSAGETSELECTEVENT(lParam) == FD_READ)
			OnGetTimeRead(wParam, lParam);
		else if (WSAGETSELECTEVENT(lParam) == FD_CLOSE)
			OnGetTimeClose(wParam, lParam);
	}

	return(TRUE);
}

























RoutingInfo::RoutingInfo()
{
	memset(hash, 0, sizeof(HASH));
	memset(guid, 0, sizeof(GLUID));
	time = 0;
	function = 0;
	id = 0;
}

RoutingInfo::~RoutingInfo()
{
}

HostInfo::HostInfo()
{
	ip = 0;
	port = 0;
	resolveType = RESOLVE_NONE;
	resolved = FALSE;
	time = 0;
	fileCount = 0; filesTotalsize = 0; hops = 0;
}

HostInfo::~HostInfo()
{
}

BOOL HostInfo::Dump(FILE *file)
{
	IP ipt;
	ipt.dword = ip;
	int rval;
	rval = fprintf(file, "%d.%d.%d.%d:%d\n", ipt.chars[0], ipt.chars[1], ipt.chars[2], ipt.chars[3], ntohs(port));
	if (rval < 0)
		return(FALSE);

	return(TRUE);
}

BOOL HostInfo::Serialize(FILE *file, BOOL store)
{
	const int serializeVersion=5;
	int fileVersion;

	if (store)
	{
		STORE_VAR(serializeVersion);

		STORE_VAR(ip);
		STORE_VAR(port);
		STORE_VAR(fileCount);
		STORE_VAR(filesTotalsize);
		STORE_VAR(hops);
		STORE_VAR(time);
		STORE_VAR(resolveType);
		STORE_VAR(resolved);
		STORE_SUBCLASS(hostName);
	}
	else
	{
		CHECK_VERSION();

		LOAD_VAR(ip);
		LOAD_VAR(port);
		LOAD_VAR(fileCount);
		LOAD_VAR(filesTotalsize);
		LOAD_VAR(hops);
		LOAD_VAR(time);
		LOAD_VAR(resolveType);
		LOAD_VAR(resolved);
		LOAD_SUBCLASS(hostName);
	}
	return(TRUE);
serializeFailed:
	return(FALSE);
}


