/******************************************************************************
*
* AntiMafia
* Copyright (C) 2002 by [epidemiC] AntiMafia
* Newtella
* Copyright (C) 2000 by Newtella Inc.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
******************************************************************************/


#define INCOMING_MAX_READ 4096
#define OUTGOING_MAX_READ 4096
#define MAX_CONNECTIONS 200

#define DEFAULT_GNUTELLAPORT 6346

#define MAX_MONITORED_SEARCHES 11
#define MAX_MONITORED_SEARCHRESULTS 10

extern UINT INCOMING_ACCEPT;
extern UINT GNUTELLANET_CONNECTED;
extern UINT GET_CONNECTED;
extern UINT PUSH_CONNECTED;
extern UINT SEND_WRITE;
extern UINT GET_READ;
extern UINT GNUTELLANET_DATA;

extern HANDLE getEvent, sendEvent, gnutellanetEvent;


enum GnutellaFunctions
{
	GNUTELLA_INIT=0x00, GNUTELLA_INITRESPONSE=0x01, GNUTELLA_PUSHREQUEST=0x40, GNUTELLA_SEARCH=0x80, GNUTELLA_SEARCHRESPONSE=0x81
};



BOOL CreateGUID(GLUID *guid);
LRESULT CALLBACK GnutellaWndFuncWrapper(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);


#include <pshpack1.h>
class GnutellaHeader
{
public:
	GLUID guid;
	BYTE function;
	BYTE ttl;
	BYTE hops;
	DWORD length;
};
class GnutellaInitResponse
{
public:
	WORD hostPort;
	DWORD hostIP;
	long fileCount;
	long filesTotalsize;
};
class GnutellaQuery
{
public:
	WORD minspeed;
};
class GnutellaQueryResponseHeader
{
public:
	BYTE numEntries;
	WORD port;
	DWORD ip;
	DWORD speed;
};
class GnutellaQueryResponseEntry
{
public:
	DWORD index;
	DWORD size;
};
class GnutellaPushRequest
{
public:
	GLUID guid;
	DWORD index;
	DWORD ip;
	WORD port;
};
#include <poppack.h>



class GnutellaWorker
{
public:
	GnutellaWorker();
	~GnutellaWorker();
	//BOOL Init();
	BOOL InitSockets();
	BOOL InitMessageWindow();
	BOOL InitListener();
	BOOL InitFiles();
	BOOL GetUploadInfo(List *list);
	BOOL GetDownloadInfo(List *list);
	BOOL Connect(const char *host);
	BOOL Search(const char *requestedsearch);
	//BOOL Disconnect(SOCKET socket);
	//BOOL Disconnect(const char *host, ConnectionType type);
	BOOL GetFile(FileInfo fi, BOOL overwrite=FALSE);
	BOOL CancelUpload(Connection *con);
	BOOL CancelDownload(Connection *con);
	BOOL ResumeDownload(Connection *con);
	BOOL RequestPush(FileInfo fi);
	BOOL Ping();
	BOOL SetListenPort();
	BOOL SendPing(Connection *con);
	BOOL EverySecond();
	BOOL EveryMinute();
	BOOL EveryTenMinutes();
	BOOL RemoveDownloadConnection(int id);
	BOOL RemoveUploadConnection(int id);
	BOOL BuildFileIndex();
	BOOL GetTime(int index);
	LRESULT CALLBACK GnutellaWndFunc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
protected:
	static void ProcBuildFileIndex(void *param);
	static void ProcResolveHostName(void *param);
	BOOL HandlePacket(Connection *con, const char *buf, int buflen);
	BOOL HandleConnect(Connection *sourcecon, const char *buf, int buflen);
	BOOL HandleConnectResponse(Connection *sourcecon, const char *buf, int buflen);
	BOOL HandleInit(Connection *sourcecon, const char *buf, int buflen);
	BOOL HandleInitResponse(Connection *sourcecon, const char *buf, int buflen);
	BOOL HandlePushRequest(Connection *sourcecon, const char *buf, int buflen);
	BOOL HandlePush(Connection *sourcecon, const char *buf, int buflen);
	BOOL HandleSearch(Connection *sourcecon, const char *buf, int buflen);
	BOOL HandleSearchResponse(Connection *sourcecon, const char *buf, int buflen);
	BOOL HandleGetRequest(Connection *sourcecon, const char *buf, int buflen);
	BOOL IsBroadcastPacket(const char *msg, unsigned msglen);
	BOOL OnSendWrite(WPARAM wParam, LPARAM lParam);
	BOOL OnGetRead(WPARAM wParam, LPARAM lParam);
	BOOL OnGnutellanetAccept(WPARAM wParam, LPARAM lParam);
	BOOL OnGnutellanetConnected(WPARAM wParam, LPARAM lParam);
	BOOL OnGetConnected(WPARAM wParam, LPARAM lParam);
	BOOL OnPushConnected(WPARAM wParam, LPARAM lParam);
	BOOL OnGnutellanetRead(WPARAM wParam, LPARAM lParam);
	BOOL OnGnutellanetWrite(WPARAM wParam, LPARAM lParam);
	BOOL OnGnutellanetClose(WPARAM wParam, LPARAM lParam);
	BOOL OnGetClose(WPARAM wParam, LPARAM lParam);
	BOOL OnSendClose(WPARAM wParam, LPARAM lParam);
	BOOL OnGetTimeConnected(WPARAM wParam, LPARAM lParam);
	BOOL OnGetTimeRead(WPARAM wParam, LPARAM lParam);
	BOOL OnGetTimeClose(WPARAM wParam, LPARAM lParam);

private:
	BOOL Connect(DWORD ip, WORD port);
	BOOL PacketNazi(const char *msg, unsigned msglen);
	//BOOL PacketSuperNazi(const char *msg, unsigned msglen);
	BOOL IsPacketComplete(const char *msg, unsigned msglen);
	BOOL CheckGIVSyntax(const char *msg, unsigned msglen);
	DWORD GetPacketLength(const char *msg, unsigned msglen);
	BOOL ContinueSendFile(Connection *con);
	BOOL ContinueGetFile(Connection *con);
	BOOL StartGet(Connection *con);
	BOOL PushFile(Connection *oldcon, GnutellaPushRequest gpr);
	DWORD MyIP();
	BOOL IsPrivateIP(DWORD ip);
public:
	List gnutellanetConnections, downloadConnections, uploadConnections;
	List foundFiles;
	List hosts;
	List receivedResults;
private:
	static FileWorker files;
	static HANDLE filesMutex;
	static List resolveHosts;
	static HANDLE resolveHostsMutex, resolveHostsAbort;
	List requestedPushes;
	List pingRoutes, searchRoutes, pushRoutes;
	WSAData wsaData;
	GLUID currentSearchGUID;
	HWND messageWindow;
	SOCKET serverSocket;
	int lastPing;

};


