/******************************************************************************
*
* AntiMafia
* Copyright (C) 2002 by [epidemiC] AntiMafia
* Newtella
* Copyright (C) 2000 by Newtella Inc.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
******************************************************************************/


#include "..\stdafx.h"
#include <string.h>
#include <stdio.h>
#include "common.h"
#include "Object.h"
#include "String.h"


String::String()
{
	data = new char[1];
	data[0] = '\0';
	length = 0;
}

String::String(const String &string2)
{
	length = string2.GetLength();
	data = new char[length+1];
	strcpy(data, string2.data);
}

String::String(const char *string2)
{
	length = strlen(string2);
	data = new char[length+1];
	strcpy(data, string2);
}

String::String(const char *string2, int len)
{
	length = len;
	data = new char[length+1];
	memcpy(data, string2, length);
	data[length] = '\0';
}

String::~String()
{
	if (data)
	{
		delete [] data;
		data = NULL;
	}
}

int String::GetLength() const
{
	return(length);
}

String::operator LPCSTR() const
{
	return(data);
}


int String::operator==(const String& string2) const
{
	return(!strcmp(data, string2.data));
}

int String::operator!=(const String& string2) const
{
	return(strcmp(data, string2.data));
}

const String& String::operator=(const String& string2)
{
	if (&string2 == this)
		return(*this);
	if (data)
		delete [] data;
	length = string2.GetLength();
	data = new char[length+1];
	if (length)
		strcpy(data, string2.data);
	else
		data[0] = '\0';

	return(*this);
}

const String& String::operator=(const char *string2)
{
	if (data)
		delete [] data;
	length = strlen(string2);
	data = new char[length+1];
	if (length)
		strcpy(data, string2);
	else
		data[0] = '\0';

	return(*this);
}

String String::operator+=(String& string2)
{
	Concat(string2);
	return(*this);
}

String String::operator+=(LPCSTR string2)
{
	Concat(String(string2));
	return(*this);
}

String String::operator+(String& string2)
{
	int newlength = length + string2.length;
	char *temp = new char[newlength+1];
	strcpy(temp, data);
	strcpy(temp+length, string2.data);
	String r(temp);
	delete [] temp;
	return(r);
}

BOOL String::Concat(const String& string2)
{
	char *tempdata = new char[length];
	memcpy(tempdata, data, GetLength());
	int templength = length + string2.length;
	if (data)
		delete [] data;
	data = new char[templength+1];
	memcpy(data, tempdata, length);
	strcpy(data+length, string2.data);
	length = templength;
	delete [] tempdata;
	return(TRUE);
}

String String::Right(unsigned int n) const
{
	String r;
	if (n > length)
		return(r);
	unsigned int pos = length-n;
	r = data+pos;
	return(r);
}

String String::Left(unsigned int n) const
{
	String r;
	if (n > length)
		return(r);
	char *copybuf = new char[n+1];
	memcpy(copybuf, data, n);
	copybuf[n] = '\0';
	r = copybuf;
	delete [] copybuf;
	return(r);
}

String String::RightOf(char chr) const
{
	int newlength;
	String r;

	if ((newlength = Strchr(chr)) == -1)
		r = "";
	else
	{
		newlength = length - newlength - 1;
		char *temp = new char[newlength+1];
		strcpy(temp, data+length-newlength);
		r = temp;
		delete [] temp;
	}

	return(r);
}

String String::RightOfLast(char chr) const
{
	int newlength;
	if ((newlength = Strrchr(chr)) == -1)
		return(String(""));
	newlength = length - newlength - 1;
	
	char *temp = new char[newlength+1];
	strcpy(temp, data+length-newlength);
	String s(temp);
	delete temp;
	return(s);
}

String String::LeftOf(char chr) const
{
	int newlength;
	if ((newlength = Strchr(chr)) == -1)
		return(*this);
	
	char *temp = new char[newlength+1];
	memcpy(temp, data, newlength);
	temp[newlength] = '\0';
	String s(temp);
	delete temp;
	return(s);
}

String String::LeftOfLast(char chr) const
{
	int newlength;
	if ((newlength = Strrchr(chr)) == -1)
		return(String(""));
	
	char *temp = new char[newlength+1];
	memcpy(temp, data, newlength);
	temp[newlength] = '\0';
	String s(temp);
	delete temp;
	return(s);
}

BOOL String::RemoveNonAlphaNums()
{
	/*
	int newLength = length;
	int t=0;
	for (DWORD i=0; i<length; i++)
	{
		if (data[i] != ' ' && !isalnum(data[i]))
		{
			t++;
			newLength--;
		}
		data[i-t+1] = data[i+1];
	}
	length = newLength;
	data[length] = '\0';
	*/
	for (DWORD i=0; i<length; i++)
	{
		if (!isalnum(data[i]) && data[i] != '.')
			data[i] = ' ';
	}

	return(TRUE);
}

BOOL String::RemoveDoubleSpaces()
{
	int newLength = length;
	int t=0;
	BOOL spaceAllowed = FALSE;
	for (DWORD i=0; i<length; i++)
	{
		if (data[i] == ' ')
		{
			if (!spaceAllowed)
			{
				t++;
				newLength--;
			}
			else
				spaceAllowed = FALSE;
		}
		else
			spaceAllowed = TRUE;
		data[i-t+1] = data[i+1];
	}
	if (data[newLength-1] == ' ')
		newLength--;
	length = newLength;
	data[length] = '\0';

	return(TRUE);
}

int String::Count(char chr) const
{
	int r=0;
	char *d=data;
	while (*d != '\0')
	{
		if (*(d++) == chr)
			r++;
	}
	return(r);
}

int String::Strchr(char chr) const
{
	if (length == 0)
		return(-1);
	char *p = data-1;
	while (p++ < data+length)
	{
		if (*p == chr)
			return(p - data);
	}
	return(-1);
}

int String::Strrchr(char chr) const
{
	if (length == 0)
		return(-1);
	char *p = data + length;
	while (--p >= data)
	{
		if (*p == chr)
			return(p - data);
	}
	return(-1);
}

int String::Find(String token) const
{
	char *s = strstr(data, token);
	if (!s)
		return(-1);
	return(s - data);
}

int String::StrStr(String token) const
{
	char *p = strstr(data, token);

	if (!p)
		return(-1);
	return(p-data);
}

int String::FindNoCase(String token) const
{
	if (!length)
		return(FALSE);
	String s1(*this), s2(token);
	s1.ToLower();
	s2.ToLower();
	return(s1.Find(s2));
}

BOOL String::ToLower()
{
	strlwr(data);
	return(TRUE);
}

BOOL String::IsHexDecString()
{
	for (DWORD i=0; i<length; i++)
	{
		if (data[i] < '0')
			return(FALSE);
		if (data[i] > 'f')
			return(FALSE);
		if (data[i] > '9' && data[i] < 'A')
			return(FALSE);
		if (data[i] > 'F' && data[i] < 'a')
			return(FALSE);
	}
	return(TRUE);
}

BOOL String::MatchWildcard(String w)
{
	String s = *this, t;
	int pos ;

	if (w.Find("*") == -1 && w.Find("?") == -1)
	{
		if (s.FindNoCase(w) == -1) return FALSE ;
		else return TRUE ;
	}

	for (;;)
	{
		if (w.Left(1) == String("?"))
		{
			w = w.Right(w.GetLength() - 1);
			if (s.GetLength() >= 1)
				s = s.Right(s.GetLength() - 1);
			else
				return(FALSE);
		}
		else if (w.Left(1) == String("*"))
		{
			w = w.Right(w.GetLength() - 1);
			if (!w.GetLength())
				return(TRUE);
			t = w.LeftOf('*');
			t = t.LeftOf('?');
			pos = s.FindNoCase(t);
			if (pos == -1)
				return(FALSE);
			s = s.Right(s.GetLength() - pos - t.GetLength());
			w = w.Right(w.GetLength() - t.GetLength());
		}
		else
		{
			t = w.LeftOf('*');
			t = t.LeftOf('?');
			if (s.Left(t.GetLength()) != t)
				return(FALSE);
			s = s.Right(s.GetLength() - t.GetLength());
			w = w.Right(w.GetLength() - t.GetLength());
		}

		//if (!s.GetLength() && !w.GetLength()) ????
		if (!s.GetLength() || !w.GetLength())
			return(TRUE);
	}
}

BOOL String::GetWord(unsigned int n, String &r) const
{
	unsigned int i;
	int start=-1, end=0;

	r = "";
	if (n == 0)
		start = 0;
	for (i=0; i<length; i++)
	{
		if (data[i] == ' ')
		{
			if (n == 0)
			{
				end = i;
				break;
			}
			else
			{
				n--;
				if (n == 0)
					start = i+1;
			}
		}
	}
	if (i == length)
	{
		if (start == -1)
			return(FALSE);
		end = i;
	}
	r = GetSubstring(start, end);

	return(r.GetLength());
}

BOOL String::GetLine(unsigned int n, String &r) const
{
	unsigned int i;
	int start=-1, end=0;

	r = "";
	if (n == 0)
		start = 0;
	for (i=0; i<length; i++)
	{
		if (data[i] == '\r')
		{
			if (n == 0)
			{
				end = i;
				break;
			}
			else
			{
				n--;
				if (data[i+1] == '\n')
					i++;
				if (n == 0)
					start = i+1;
			}
		}
	}
	if (i == length)
	{
		if (start == -1)
			return(FALSE);
		end = i;
	}
	r = GetSubstring(start, end);

	return(r.GetLength());
}

String String::GetSubstring(int start, int end) const
{
	char *tempstr = new char[end-start+1];
	memcpy(tempstr, data+start, end-start);
	tempstr[end-start] = 0;
	String r(tempstr);
	delete [] tempstr;
	return(r);
}

BOOL String::Serialize(FILE *file, BOOL store)
{
	const int serializeVersion=2;
	int fileVersion;

	if (store)
	{
		STORE_VAR(serializeVersion);

		STORE_VAR(length);
		STORE_BUFFER(data, length);
	}
	else
	{
		CHECK_VERSION();

		LOAD_VAR(length);
		delete [] data;
		data = NULL;
		data = new char[length+1];
		LOAD_BUFFER(data, length);
		data[length] = '\0';
	}
	return(TRUE);
serializeFailed:
	if (!store)
	{
		delete [] data;
		data = new char[1];
		data[0] = '\0';
	}
	return(FALSE);
}

